import networkx

import pyvex
import simuvex

class Blade(object):
    '''
    Blade is a light-weight program slicer that works with networkx DiGraph containing SimIRSBs.
    It is meant to be used in angr for small or on-the-fly analyses.
    '''
    def __init__(self, graph, dst_run, dst_stmt_idx, direction='backward', project=None):
        self._graph = graph
        self._dst_run = dst_run
        self._dst_stmt_idx = dst_stmt_idx

        self._slice = networkx.DiGraph()

        if not self._in_graph(self._dst_run):
            raise AngrBladeError("The specified SimRun %s doesn't exist in graph.")

        self._p = project

        self._run_cache = { }

        self._traced_runs = set()

        if direction == 'backward':
            self._backward_slice()
        elif direction == 'forward':
            raise AngrBladeError('Forward slicing is not implemented yet')
        else:
            raise AngrBladeError("Unknown slicing direction %s", direction)

    def _get_run(self, v):
        if isinstance(v, simuvex.SimIRSB) or isinstance(v, simuvex.SimProcedure):
            return v

        elif type(v) in (int, long):
            # Generate an IRSB from self._project

            if v in self._run_cache:
                return self._run_cache[v]

            if self._p:
                irsb = self._p.factory.block(v).vex
                self._run_cache[v] = irsb
                return irsb
            else:
                raise AngrBladeError("Project must be specified if you give me all addresses for SimRuns")

        else:
            raise AngrBladeError('Unsupported SimRun argument type %s', type(v))

    def _normalize(self, v):
        if isinstance(v, simuvex.SimIRSB) or isinstance(v, simuvex.SimProcedure):
            if type(self._graph.nodes()[0]) in (int, long):
                return v.addr
            else:
                return v
        elif type(v) in (int, long):
            return v
        else:
            raise AngrBladeError('Unsupported SimRun argument type %s', type(v))

    def _in_graph(self, v):
        return self._normalize(v) in self._graph


    @property
    def slice(self):
        return self._slice

    def _inslice_callback(self, stmt_idx, stmt, infodict):
        tpl = (infodict['irsb_addr'], stmt_idx)
        if 'prev' in infodict and infodict['prev']:
            prev = infodict['prev']
            self._slice.add_edge(tpl, prev)
        else:
            self._slice.add_node(tpl)

        infodict['prev'] = tpl

    def _backward_slice(self):
        '''
        Backward slicing.

        We support the following IRStmts:
        # WrTmp
        # Put

        We support the following IRExprs:
        # Get
        # RdTmp
        # Const

        :return:
        '''

        temps = set()
        regs = set()

        # Retrieve the target: are we slicing from a register(IRStmt.Put), or a temp(IRStmt.WrTmp)?
        stmts = self._get_run(self._dst_run).irsb.statements

        if self._dst_stmt_idx != -1:
            dst_stmt = stmts[self._dst_stmt_idx]

            if type(dst_stmt) is pyvex.IRStmt.Put:
                regs.add(dst_stmt.offset)
            elif type(dst_stmt) is pyvex.IRStmt.WrTmp:
                temps.add(dst_stmt.tmp)
            else:
                raise AngrBladeError('Incorrect type of the specified target statement. We only support Put and WrTmp.')

        else:
            next_expr = self._get_run(self._dst_run).irsb.next

            if type(next_expr) is pyvex.IRExpr.RdTmp:
                temps.add(next_expr.tmp)
            elif type(next_expr) is pyvex.IRExpr.Const:
                # A const doesn't rely on anything else!
                pass
            else:
                raise AngrBladeError('Unsupported type for irsb.next: %s' % type(next_expr))

            # Then we gotta start from the very last statement!
            self._dst_stmt_idx = len(stmts) - 1

        slicer = simuvex.SimSlicer(stmts, temps, regs,
                                   inslice_callback=self._inslice_callback,
                                   inslice_callback_infodict={
                                       'irsb_addr':  self._get_run(self._dst_run).addr
                                   })
        regs = slicer.final_regs

        prev = slicer.inslice_callback_infodict['prev']

        if regs:
            predecessors = self._graph.predecessors(self._normalize(self._dst_run))

            for p in predecessors:
                if p not in self._traced_runs:
                    self._traced_runs.add(p)
                    self._backward_slice_recursive(p, regs, prev)

    def _backward_slice_recursive(self, run, regs, prev):
        temps = set()
        regs = regs.copy()

        stmts = self._get_run(run).irsb.statements

        # Initialize the temps set with whatever in the `next` attribute of this irsb
        next_expr = self._get_run(run).irsb.next
        if type(next_expr) is pyvex.IRExpr.RdTmp:
            temps.add(next_expr.tmp)

        slicer = simuvex.SimSlicer(stmts, temps, regs,
                                   inslice_callback=self._inslice_callback,
                                   inslice_callback_infodict={
                                       'irsb_addr' : self._get_run(run).addr,
                                       'prev' : prev
                                   })
        regs = slicer.final_regs

        prev = slicer.inslice_callback_infodict['prev']

        if regs:
            predecessors = self._graph.predecessors(self._normalize(run))

            for p in predecessors:
                if p not in self._traced_runs:
                    self._traced_runs.add(p)
                    self._backward_slice_recursive(p, regs, prev)

    #
    # Backward slice IRStmt handlers
    #

    def _backward_handler_stmt_WrTmp(self, stmt, temps, regs):
        tmp = stmt.tmp

        if tmp not in temps:
            return False

        temps.remove(tmp)

        self._backward_handler_expr(stmt.data, temps, regs)

        return True

    def _backward_handler_stmt_Put(self, stmt, temps, regs):
        reg = stmt.offset

        if reg in regs:
            regs.remove(reg)

            self._backward_handler_expr(stmt.data, temps, regs)

            return True

        else:
            return False

    #
    # Backward slice IRExpr handlers
    #

    def _backward_handler_expr(self, expr, temps, regs):
        funcname = "_backward_handler_expr_%s" % type(expr).__name__
        in_slice = False
        if hasattr(self, funcname):
            in_slice = getattr(self, funcname)(expr, temps, regs)

        return in_slice

    def _backward_handler_expr_RdTmp(self, expr, temps, regs):
        tmp = expr.tmp

        temps.add(tmp)

    def _backward_handler_expr_Get(self, expr, temps, regs):
        reg = expr.offset

        regs.add(reg)

    def _backward_handler_expr_Load(self, expr, temps, regs):
        addr = expr.addr

        if type(addr) is pyvex.IRExpr.RdTmp:
            # FIXME: Process other types
            self._backward_handler_expr(addr, temps, regs)

from .errors import AngrBladeError



import sys
import contextlib
import functools

RESULT_ERROR = "An error occured"
RESULT_NONE = "No result"

class AnalysisLogEntry(object):
    def __init__(self, message, exc_info=False):
        if exc_info:
            (e_type, value, traceback) = sys.exc_info()
            self.exc_type = e_type
            self.exc_value = value
            self.exc_traceback = traceback
        self.message = message

    def __getstate__(self):
        return str(self.__dict__.get("exc_type")), \
               str(self.__dict__.get("exc_value")), \
               str(self.__dict__.get("exc_traceback"))

    def __setstate__(self, s):
        self.exc_type, self.exc_value, self.exc_traceback = s


class AnalysisMeta(type):
    """
    This metaclass is Yan being too clever with analysis creation, and will probably
    come back to bite us in the ass. Basically, it replaces the Analysis' __init__
    with the base __init__, which yanks the project and the fail_fast setting out
    from the arguments. This lets us avoid the three worse alternatives:

        1. override __new__, which doesn't work because you still can't pull the
           args out
        2. make the analyses implement something other than __init__, which will
           reduce pylint's effectiveness in regards to the defined-outside-init stuff
        3. make the analyses handle the project and fail_fast args and pass them on
           to Analysis.__init__
    """

    def __new__(mcs, name, bases, d):
        if name == 'Analysis':
            d['__init__'] = d['__core_init__']
        else:
            d['__analysis_init__'] = d['__init__']
            del d['__init__']

        t = type.__new__(mcs, name, bases, d)
        if name != 'Analysis':
            registered_analyses[d.get('__analysis_name__', name)] = t
        return t


# function that produces unpredictable results that should appease pylint's
# static analysis and stop giving us those awful errors!!!!

def dummy_func(*args, **kwargs):
    return args + list(kwargs)

class Analyses(object):
    """
    This class contains functions for all the registered and runnable analyses,
    """

    def _analysis(self, analysis, *args, **kwargs):
        fail_fast = kwargs.pop('fail_fast', False)

        # Call __init__ of chosen analysis
        a = analysis(self._p, fail_fast, *args, **kwargs)

        return a

    def __init__(self, p):
        """
        Creates an Analyses object

        @param p: the angr.Project object
        @param analysis_results: the result cache
        """
        self._p = p

        # Appease pylint's static analysis
        self.CFG = dummy_func
        self.VFG = dummy_func
        self.Veritesting = dummy_func
        self.DDG = dummy_func
        self.CDG = dummy_func
        self.BackwardSlice = dummy_func
        self.BoyScout = dummy_func
        self.GirlScout = dummy_func

        for analysis_name,analysis in registered_analyses.items():
            partial = functools.partial(self._analysis, analysis)
            partial.__doc__ = analysis_name + ' analysis'
            if analysis.__doc__:
                partial.__doc__ += analysis.__doc__
            if analysis.__analysis_init__.__doc__:
                partial.__doc__ += analysis.__analysis_init__.__doc__
            setattr(self, analysis_name, partial)

    def __getstate__(self):
        return self._p

    def __setstate__(self, s):
        self.__init__(s)


registered_analyses = {}

class Analysis(object):
    __metaclass__ = AnalysisMeta

    def __core_init__(self, project, fail_fast, *args, **kwargs):
        # pylint:disable=attribute-defined-outside-init
        self.named_errors = {}
        self.errors = []
        self.log = []

        self._fail_fast = fail_fast
        self._p = project

        self.result = RESULT_NONE

        if kwargs.pop('do_analysis', True):
            self.__analysis_init__(*args, **kwargs)  # pylint:disable=no-member

    def post_load(self):
        pass

    @contextlib.contextmanager
    def _resilience(self, name=None, exception=Exception):
        try:
            yield
        except exception:  # pylint:disable=broad-except
            if self._fail_fast:
                raise
            else:
                error = AnalysisLogEntry("exception occurred", exc_info=True)
                if name is None:
                    self.errors.append(error)
                else:
                    self.named_errors[name] = error

    def _log(self, event):
        '''

        :return:
        '''
        # TODO: This function is not properly designed nor implemented!
        le = AnalysisLogEntry(event)
        self.log.append(le)

    def _checkpoint(self):
        pass

    def copy(self):
        return self.__class__(self._p, self._fail_fast, do_analysis=False)



from .datagraph_meta import DataGraphMeta, DataGraphError
import logging
import networkx
import collections
import simuvex

l = logging.getLogger(name="angr.analyses.vsa_ddg")


class VSA_DDG(DataGraphMeta):
    """
    A Data dependency graph based on VSA states.
    That means we don't (and shouldn't) expect any symbolic expressions.
    """

    def __init__(self, start_addr, interfunction_level=0,
                 context_sensitivity_level=2, keep_addrs=False):
        """
        @start_addr: the address where to start the analysis (typically, a
        function's entry point)

        @interfunction_level and @context_sensitivity_level have the same
        meaning as in the VFG analysis.

        @keep_addrs: whether we keep set of addresses as edges in the graph, or
        just the cardinality of the sets, which can be used as a "weight".

        Returns: a NetworkX graph representing data dependencies.
        """

        self._startnode = None # entry point of the analyzed function
        self._vfg = self._p.analyses.VFG(function_start=start_addr,
                                         interfunction_level=interfunction_level,
                                         context_sensitivity_level=context_sensitivity_level)
        self.graph = networkx.DiGraph()
        self.keep_addrs = keep_addrs

        self._simproc_map = {}
        self._imarks = {}

        # Get the first node
        self._startnode = self._vfg_node(start_addr)

        if self._startnode is None:
            raise DataGraphError ("No start node :(")

        # We explore one path at a time
        self._branch({}, self._startnode)

    def _make_block(self, irsb, live_defs):
        return Block(irsb, live_defs, self.graph, self.keep_addrs)

class Block(object):
    """
    Defs and uses in a block.
    """
    def __init__(self, irsb, live_defs, graph, keep_addrs):
        """
        @irsb: a SimIRSB object

        @live_defs: a dict {addr:stmt} containing the definitions from previous
        blocks that are still live at this point, where addr is a tuple
        representing a normalized addr (see simuvex/plugins/abstract_memory.py for more)

        @keep_addrs: edges in the graph may either be labelled with the
        cardinality of the set of addresses involved, or the set of addresses itself.

        """
        self.irsb = irsb
        self.live_defs= live_defs
        self.graph = graph
        self.keep_addrs = keep_addrs
        self._imarks = {}

        # A repeating block is a block creating an already existing edge in the
        # graph, which is where we want to stop analyzing a specific path
        self._read_edge = False  # The block causes a read edge in the graph
        self._new = False  # There is at least one new read edge
        if isinstance(irsb, simuvex.SimProcedure):
            # TODO: track reads and writes in SimProcedures
            self._track_actions(-1, irsb.successors[0].log.actions)
        else:
            for st in self.irsb.statements:
                self._imarks[(self.irsb.addr, st.stmt_idx)] = st.imark.addr
                self._track_actions(st.stmt_idx, st.actions)

    def _track_actions(self, stmt_idx, a_list):
        for a in a_list:
            if a.type == "mem":
                addr_list = set(self.irsb.initial_state.memory.normalize_address(a.addr.ast))

                node = (self.irsb.addr, stmt_idx)

                prevdefs = self._def_lookup(addr_list)

                if a.action == "read":
                    for prev_node, label in prevdefs.iteritems():
                        self._read_edge = True
                        self._add_edge(prev_node, node, label)

                if a.action == "write":
                    self._kill(addr_list, node)

    def _def_lookup(self, addr_list):
        """
        This is a backward lookup in the previous defs.
        @addr_list is a list of normalized addresses.
        Note that, as we are using VSA, it is possible that @a is affected by
        several definitions.
        Returns: a dict {stmt:label} where label is the number of individual
        addresses of @addr_list (or the actual set of addresses depending on the
        keep_addrs flag) that are definted by stmt.
        """

        if self.keep_addrs is True:
            prevdefs = collections.defaultdict(set)
        else:
            prevdefs = collections.defaultdict(int)  # default value of int is 0

        for addr in addr_list:
            if addr in self.live_defs.keys():
                stmt = self.live_defs[addr]
                # Label edges with cardinality or actual sets of addresses
                if self.keep_addrs is True:
                    prevdefs[stmt].add(addr)
                else:
                    prevdefs[stmt] = prevdefs[stmt] + 1
        return prevdefs

    def _kill(self, addr_list, stmt):
        """
        Kill previous defs. @addr_list is a list of normalized addresses
        """

        # Case 1: address perfectly match, we kill
        # Case 2: a is a subset of the original address
        # Case 3: a is a superset of the original address
        for addr in addr_list:
            self.live_defs[addr] = stmt
            l.debug("XX Stmt (0x%x, %d) kills addr %s" % (stmt[0], stmt[1], repr(addr)))

    def _add_edge(self, s_a, s_b, label):
        """
         Add an edge in the graph from @s_a to statment @s_b, where @s_a and
         @s_b are tuples of statements of the form (irsb_addr, stmt_idx)
        """
        # Is that edge already in the graph ?
        # If at least one is new, then we are not redoing the same path again
        if (s_a, s_b) not in self.graph.edges():
            self.graph.add_edge(s_a, s_b, label=label)
            self._new = True
            l.info("New edge from (0x%x, %d) --> (0x%x, %d)" %
               (s_a[0], s_a[1],
                s_b[0], s_b[1]))


    @property
    def stop(self):
        """
        If this block contains a read that is not creating new edges in the graph,
        then we are looping and we should stop the analysis.
        """
        return self._read_edge and not self._new





#!/usr/bin/env python

# pylint: disable=W0703

import os
import types
import logging
import weakref

import cle
import simuvex
import archinfo

l = logging.getLogger("angr.project")

projects = weakref.WeakValueDictionary()
def fake_project_unpickler(name):
    if name not in projects:
        raise AngrError("Project %s has not been opened." % name)
    return projects[name]
fake_project_unpickler.__safe_for_unpickling__ = True

def deprecated(f):
    def deprecated_wrapper(*args, **kwargs):
        print "ERROR: FUNCTION %s IS DEPRECATED. PLEASE UPDATE YOUR CODE." % f
        return f(*args, **kwargs)
    return deprecated_wrapper

class Project(object):
    """
    This is the main class of the Angr module. It is meant to contain a set of
    binaries and the relationships between them, and perform analyses on them.
    """

    def __init__(self, thing,
                 default_analysis_mode=None,
                 ignore_functions=None,
                 use_sim_procedures=True,
                 exclude_sim_procedures_func=None,
                 exclude_sim_procedures_list=(),
                 arch=None, simos=None,
                 load_options=None,
                 parallel=False,
                 support_selfmodifying_code=False):
        """
        This constructs a Project object.

        Arguments:
         @param thing
             the path to the main executable object to analyze, or a CLE Loader object
         @param default_analysis_mode
             the mode of analysis to use by default. Defaults to 'symbolic'.
         @param ignore_functions
             a list of function names that, when imported from shared libraries,
             should never be stepped into in analysis (calls will return an
             unconstrained value)
         @param use_sim_procedure
             whether to replace resolved dependencies for which simprocedures
             are available with said simprocedures
         @param exclude_sim_procedures_func
             a function that, when passed a function name, returns whether
             or not to wrap it with a simprocedure
         @param exclude_sim_procedures_list
             a list of functions to *not* wrap with simprocedures
         @param arch
             optional target architecture (auto-detected otherwise)
             in the form of an archinfo.Arch or a string
         @param simos
             a SimOS class to use for this project
         @param load_options
             a dict of keyword arguments to the CLE loader. See CLE's docs.
             e.g., { 'auto_load_libs': False,
                     'skip_libs': 'ld.so.2',
                     'lib_opts': {
                       'libc.so.6': {
                         'custom_base_addr': 0x55555400
                       }
                     }
                   }
         @param parallel
             whether to use parallel processing analyzing this binary
         @param support_selfmodifying_code
             Whether we support self-modifying code. When enabled, Project.sim_block() will try to read code from the
             given state, not only from the initial memory regions.
        """

        # Step 1: Load the binary
        if isinstance(thing, cle.Loader):
            self.loader = thing
            self.filename = self.loader._main_binary_path
        elif not isinstance(thing, (unicode, str)) or not os.path.exists(thing) or not os.path.isfile(thing):
            raise Exception("Not a valid binary file: %s" % repr(thing))
        else:
            # use angr's loader, provided by cle
            l.info("Loading binary %s", thing)
            self.filename = thing
            if load_options is None: load_options = {}
            self.loader = cle.Loader(self.filename, **load_options)

        # Step 2: determine its CPU architecture, ideally falling back to CLE's guess
        if isinstance(arch, str):
            self.arch = archinfo.arch_from_id(arch) # may raise ArchError, let the user see this
        elif isinstance(arch, archinfo.Arch):
            self.arch = arch
        elif arch is None:
            self.arch = self.loader.main_bin.arch
        else:
            raise ValueError("Invalid arch specification.")

        # Step 3: Set some defaults and set the public and private properties
        if not default_analysis_mode:
            default_analysis_mode = 'symbolic'
        if not ignore_functions:
            ignore_functions = []

        if isinstance(exclude_sim_procedures_func, types.LambdaType):
            l.warning("Passing a lambda type as the exclude_sim_procedures_func argument to Project causes the resulting object to be un-serializable.")

        self._sim_procedures = {}
        self._default_analysis_mode = default_analysis_mode
        self._exclude_sim_procedures_func = exclude_sim_procedures_func
        self._exclude_sim_procedures_list = exclude_sim_procedures_list
        self._should_use_sim_procedures = use_sim_procedures
        self._parallel = parallel
        self._support_selfmodifying_code = support_selfmodifying_code
        self._ignore_functions = ignore_functions
        self._extern_obj = AngrExternObject()
        self.loader.add_object(self._extern_obj)

        self._cfg = None
        self._vfg = None
        self._cdg = None

        self.entry = self.loader.main_bin.entry
        self.factory = AngrObjectFactory(self)
        self.analyses = Analyses(self)
        self.surveyors = Surveyors(self)

        projects[self.filename] = self

        # Step 4: Register simprocedures as appropriate for library functions
        self._use_sim_procedures()

        # Step 5: determine the host OS and perform additional initialization
        # in the SimOS constructor
        if isinstance(simos, type) and issubclass(simos, SimOS):
            self._simos = simos(self) #pylint:disable=invalid-name
        elif simos is None:
            self._simos = os_mapping[self.loader.main_bin.os](self)
        else:
            raise ValueError("Invalid OS specification or non-matching architecture.")

    #
    # Public methods
    #

    def hook(self, addr, func, length=0, kwargs=None):
        """
         Hook a section of code with a custom function.

         @param addr        The address to hook
         @param func        A python function or SimProcedure class that will perform an action when
                            execution reaches the hooked address
         @param length      How many bytes you'd like to skip over with your hook. Can be zero.
         @param kwargs      A dictionary of keyword arguments to be passed to your function or
                            your SimProcedure's run function.

         If func is a function, it takes a SimState and the given kwargs. It can return nothing
         (None), in which case it will generate a single exit to the instruction at addr+length,
         or it can return an array of successor states.

         If func is a SimProcedure, it will be run instead of a SimBlock at that address.

         If length is zero, the block at the hooked address will be executed immediately
         after the hook function.
        """

        if self.is_hooked(addr):
            l.warning("Address is already hooked [hook(%#x, %s)]", addr, func)
            return

        if kwargs is None: kwargs = {}

        if isinstance(func, type):
            proc = func
        elif hasattr(func, '__call__'):
            proc = simuvex.procedures.stubs.UserHook.UserHook
            kwargs = {'user_func': func, 'user_kwargs': kwargs, 'default_return_addr': addr+length}
        else:
            raise AngrError("%s is not a valid object to execute in a hook", func)

        self._sim_procedures[addr] = (proc, kwargs)

    def is_hooked(self, addr):
        return addr in self._sim_procedures

    def unhook(self, addr):
        if not self.is_hooked(addr):
            l.warning("Address %#x not hooked", addr)
            return

        del self._sim_procedures[addr]

    #
    # Pickling
    #

    def __getstate__(self):
        try:
            factory, analyses, surveyors = self.factory, self.analyses, self.surveyors
            self.factory, self.analyses, self.surveyors = None, None, None
            return dict(self.__dict__)
        finally:
            self.factory, self.analyses, self.surveyors = factory, analyses, surveyors

    def __setstate__(self, s):
        self.__dict__.update(s)
        self.factory = AngrObjectFactory(self)
        self.analyses = Analyses(self)
        self.surveyors = Surveyors(self)

    #
    # Private project stuff for simprocedures
    #

    def _should_exclude_sim_procedure(self, f):
        return (f in self._exclude_sim_procedures_list) or \
               ( self._exclude_sim_procedures_func is not None and \
                 self._exclude_sim_procedures_func(f)
               )

    def _find_sim_libraries(self):
        """ Look for libaries that we can replace with their simuvex
        simprocedures counterpart
        This function returns the list of libraries that were found in simuvex
        """
        simlibs = []

        for lib_name in self.loader.requested_objects:
            # Hack that should go somewhere else:
            if lib_name == 'libc.so.0':
                lib_name = 'libc.so.6'

            if lib_name == 'ld-uClibc.so.0':
                lib_name = 'ld-uClibc.so.6'

            if lib_name not in simuvex.procedures.SimProcedures:
                l.debug("There are no simprocedures for library %s :(", lib_name)
            else:
                simlibs.append(lib_name)

        return simlibs

    def _use_sim_procedures(self):
        """ Use simprocedures where we can """
        libs = self._find_sim_libraries()
        for obj in self.loader.all_objects:
            unresolved = []
            for reloc in obj.imports.itervalues():
                func = reloc.symbol
                if not func.is_function:
                    continue
                elif func.name in self._ignore_functions:
                    unresolved.append(func)
                    continue
                elif self._should_exclude_sim_procedure(func.name):
                    continue

                elif self._should_use_sim_procedures:
                    for lib in libs:
                        simfun = simuvex.procedures.SimProcedures[lib]
                        if func.name in simfun:
                            l.info("[R] %s:", func.name)
                            l.debug("\t -> matching SimProcedure in %s :)", lib)
                            self.set_sim_procedure(obj, func.name, simfun[func.name], None)
                            break
                    else: # we could not find a simprocedure for this function
                        if not func.resolved:   # the loader couldn't find one either
                            unresolved.append(func)
                # in the case that simprocedures are off and an object in the PLT goes
                # unresolved, we still want to replace it with a retunconstrained.
                elif not func.resolved and func.name in obj.jmprel:
                    unresolved.append(func)

            for func in unresolved:
                # Don't touch weakly bound symbols, they are allowed to go unresolved
                if func.is_weak:
                    continue
                l.info("[U] %s", func.name)
                procedure = simuvex.SimProcedures['stubs']['NoReturnUnconstrained']
                if func.name not in procedure.use_cases:
                    procedure = simuvex.SimProcedures['stubs']['ReturnUnconstrained']
                self.set_sim_procedure(obj, func.name, procedure, {'resolves': func.name})

    def set_sim_procedure(self, binary, func_name, sim_proc, kwargs=None):
        """
         Use a simprocedure to resolve a dependency in a binary.

         @param binary      The CLE binary whose dependency is to be resolve
         @param func_name   The name of the dependency to resolve
         @param sim_proc    The class of the SimProcedure to use
         @param kwargs      An optional dictionary of arguments to be passed
                            to the simprocedure's run() method.
        """
        if kwargs is None: kwargs = {}
        ident = sim_proc.__module__ + '.' + sim_proc.__name__
        if 'resolves' in kwargs:
            ident += '.' + kwargs['resolves']
        pseudo_addr = self._extern_obj.get_pseudo_addr(ident)
        binary.set_got_entry(func_name, pseudo_addr)

        if not self.is_hooked(pseudo_addr):     # Do not add duplicate simprocedures
            self.hook(pseudo_addr, sim_proc, kwargs=kwargs)
            l.debug("\t -> setting SimProcedure with pseudo_addr 0x%x...", pseudo_addr)

from .errors import AngrError
from .factory import AngrObjectFactory
from .simos import SimOS, os_mapping
from .extern_obj import AngrExternObject
from .analysis import Analyses
from .surveyor import Surveyors



#!/usr/bin/env python

import logging
l = logging.getLogger("angr.helpers")


def once(f):
    name = f.__name__

    def func(self, *args, **kwargs):
        if len(args) + len(kwargs) == 0:
            if hasattr(self, "_" + name):
                return getattr(self, "_" + name)

            a = f(self, *args, **kwargs)
            setattr(self, "_" + name, a)
            return a
        else:
            return f(self, *args, **kwargs)
    func.__name__ = f.__name__
    return func



import logging
import networkx
import string

import claripy

l = logging.getLogger(name="angr.functionmanager")

class Function(object):
    '''
    A representation of a function and various information about it.
    '''
    def __init__(self, function_manager, addr, name=None, syscall=False):
        '''
        Function constructor

        @param addr             The address of the function
        @param name             (Optional) The name of the function
        @param syscall          (Optional) Whether this function is a sycall or not
        '''
        self._transition_graph = networkx.DiGraph()
        self._local_transition_graph = None

        self._ret_sites = set()
        self._call_sites = {}
        self._retn_addr_to_call_site = {}
        self._addr = addr
        self._function_manager = function_manager
        self.name = name
        self.is_syscall = syscall

        if self.name is None:
            # Try to get a name from project.loader
            self.name = self._function_manager._project.loader.find_symbol_name(addr)
            if self.name is None:
                self.name = 'sub_%x' % addr

        # Register offsets of those arguments passed in registers
        self._argument_registers = []
        # Stack offsets of those arguments passed in stack variables
        self._argument_stack_variables = []

        # These properties are set by VariableManager
        self._bp_on_stack = False
        self._retaddr_on_stack = False

        self._sp_delta = 0

        # Calling convention
        self.cc = None

        # Whether this function returns or not. `None` means it's not determined yet
        self.returning = None

        self.prepared_registers = set()
        self.prepared_stack_variables = set()
        self.registers_read_afterwards = set()
        self.blocks = { addr }

    @property
    def has_unresolved_jumps(self):
        for addr in self._transition_graph.nodes():
            if addr in self._function_manager._cfg._unresolved_indirect_jumps:
                b = self._function_manager._project.factory.block(addr)
                if b.vex.jumpkind == 'Ijk_Boring':
                    return True
        return False

    @property
    def has_unresolved_calls(self):
        for addr in self._transition_graph.nodes():
            if addr in self._function_manager._cfg._unresolved_indirect_jumps:
                b = self._function_manager._project.factory.block(addr)
                if b.vex.jumpkind == 'Ijk_Call':
                    return True
        return False

    @property
    def operations(self):
        '''
        All of the operations that are done by this functions.
        '''
        operations = [ ]
        for b in self.basic_blocks:
            if b in self._function_manager._project.loader.memory:
                try:
                    operations.extend(self._function_manager._project.factory.block(b).vex.operations)
                except AngrTranslationError:
                    continue
        return operations

    @property
    def code_constants(self):
        '''
        All of the constants that are used by this functions's code.
        '''
        # TODO: remove link register values
        constants = [ ]
        for b in self.basic_blocks:
            if b in self._function_manager._project.loader.memory:
                try:
                    constants.extend(v.value for v in self._function_manager._project.factory.block(b).vex.constants)
                except AngrTranslationError:
                    continue
        return constants

    def string_references(self, minimum_length=1):
        """
        ALl of the constant string reference used by this function
        :param minimum_length: the minimum length of strings to find (default is 1)
        :return: a list of tuples of (address, string) where is address is the location of the string in memory
        """
        strings = []
        memory = self._function_manager._project.loader.memory

        # get known instruction addresses and call targets
        # these addresses cannot be string references, but show up frequently in the runtime values
        known_executable_addresses = set()
        for b in self.basic_blocks:
            if b in memory:
                sirsb = self._function_manager._project.factory.block(b)
                known_executable_addresses.update(sirsb.instruction_addrs)
        for node in self._function_manager._cfg.nodes():
            known_executable_addresses.add(node.addr)

        # loop over all local runtime values and check if the value points to a printable string
        for addr in self.partial_local_runtime_values:
            if not isinstance(addr, claripy.fp.FPV) and addr in memory:
                # check that the address isn't an pointing to known executable code
                # and that it isn't an indirect pointer to known executable code
                try:
                    possible_pointer = memory.read_addr_at(addr)
                    if addr not in known_executable_addresses and possible_pointer not in known_executable_addresses:
                        # build string
                        stn = ""
                        offset = 0
                        current_char = memory[addr + offset]
                        while current_char in string.printable:
                            stn += current_char
                            offset += 1
                            current_char = memory[addr + offset]

                        # check that the string was a null terminated string with minimum length
                        if current_char == "\x00" and len(stn) >= minimum_length:
                            strings.append((addr, stn))
                except KeyError:
                    pass
        return strings

    @property
    def partial_local_runtime_values(self):
        """
        Tries to find all runtime values of this function which do not come from inputs.
        These values are generated by starting from a blank state and reanalyzing the basic blocks once each.
        Function calls are skipped, and back edges are never taken so these values are often unreliable,
        This function is good at finding simple constant addresses which the function will use or calculate.
        :return: a set of constants
        """
        constants = set()

        if not self._function_manager._project.loader.main_bin.contains_addr(self.startpoint):
            return constants

        # reanalyze function with a new initial state (use persistent registers)
        initial_state = self._function_manager._cfg.get_any_irsb(self.startpoint).initial_state
        fresh_state = self._function_manager._project.factory.blank_state(mode="fastpath")
        for reg in initial_state.arch.persistent_regs + ['ip']:
            fresh_state.registers.store(reg, initial_state.registers.load(reg))

        # process the nodes in a breadth-first order keeping track of which nodes have already been analyzed
        analyzed = set()
        q = [fresh_state]
        analyzed.add(fresh_state.se.any_int(fresh_state.ip))
        while len(q) > 0:
            state = q.pop()
            # don't trace into simprocedures
            if self._function_manager._project.is_hooked(state.se.any_int(state.ip)):
                continue
            # don't trace outside of the binary
            if not self._function_manager._project.loader.main_bin.contains_addr(state.se.any_int(state.ip)):
                continue

            # get runtime values from logs of successors
            p = self._function_manager._project.factory.path(state)
            p.step()
            for succ in p.next_run.flat_successors + p.next_run.unsat_successors:
                for a in succ.log.actions:
                    for ao in a.all_objects:
                        if not isinstance(ao.ast, claripy.Base):
                            constants.add(ao.ast)
                        elif not ao.ast.symbolic:
                            constants.add(succ.se.any_int(ao.ast))

                # add successors to the queue to analyze
                if not succ.se.symbolic(succ.ip):
                    succ_ip = succ.se.any_int(succ.ip)
                    if succ_ip in self.basic_blocks and succ_ip not in analyzed:
                        analyzed.add(succ_ip)
                        q.insert(0, succ)

            # force jumps to missing successors
            # (this is a slightly hacky way to force it to explore all the nodes in the function)
            missing = set(self.transition_graph.successors(state.se.any_int(state.ip))) - analyzed
            for succ_addr in missing:
                l.info("Forcing jump to missing successor: 0x%x", succ_addr)
                if succ_addr not in analyzed:
                    all_successors = p.next_run.unconstrained_successors + p.next_run.flat_successors + \
                                     p.next_run.unsat_successors
                    if len(all_successors) > 0:
                        # set the ip of a copied successor to the successor address
                        succ = all_successors[0].copy()
                        succ.ip = succ_addr
                        analyzed.add(succ_addr)
                        q.insert(0, succ)
                    else:
                        l.warning("Could not reach successor: 0x%x", succ_addr)

        return constants

    @property
    def runtime_values(self):
        '''
        All of the concrete values used by this function at runtime (i.e., including passed-in arguments and global values).
        '''
        constants = set()
        for b in self.basic_blocks:
            for sirsb in self._function_manager._cfg.get_all_irsbs(b):
                for s in sirsb.successors + sirsb.unsat_successors:
                    for a in s.log.actions:
                        for ao in a.all_objects:
                            if not isinstance(ao.ast, claripy.Base):
                                constants.add(ao.ast)
                            elif not ao.ast.symbolic:
                                constants.add(s.se.any_int(ao.ast))
        return constants

    def __contains__(self, val):
        if isinstance(val, (int, long)):
            return val in self._transition_graph
        else:
            return False

    def __str__(self):
        if self.name is None:
            s = 'Function [0x%x]\n' % (self._addr)
        else:
            s = 'Function %s [0x%x]\n' % (self.name, self._addr)
        s += '  Syscall: %s\n' % self.is_syscall
        s += '  SP difference: %d\n' % self.sp_delta
        s += '  Has return: %s\n' % self.has_return
        s += '  Returning: %s\n' % ('Unknown' if self.returning is None else self.returning)
        s += '  Arguments: reg: %s, stack: %s\n' % \
            (self._argument_registers,
             self._argument_stack_variables)
        s += '  Blocks: [%s]\n' % ", ".join([hex(i) for i in self.blocks])
        s += "  Calling convention: %s" % self.cc
        return s

    def __repr__(self):
        if self.name is None:
            return '<Function 0x%x>' % (self._addr)
        else:
            return '<Function %s (0x%x)>' % (self.name, self._addr)

    @property
    def startpoint(self):
        return self._addr

    @property
    def endpoints(self):
        return list(self._ret_sites)

    def clear_transition_graph(self):
        self.blocks = { self._addr }
        self._transition_graph = networkx.DiGraph()
        self._transition_graph.add_node(self._addr)
        self._local_transition_graph = None

    def transit_to(self, from_addr, to_addr):
        '''
        Registers an edge between basic blocks in this function's transition graph

        @param from_addr            The address of the basic block that control
                                    flow leaves during this transition
        @param to_addr              The address of the basic block that control
                                    flow enters during this transition
        '''

        self.blocks.add(from_addr)
        self.blocks.add(to_addr)

        self._transition_graph.add_edge(from_addr, to_addr, type='transition')

    def call_to(self, from_addr, to_addr, return_target, syscall=False):
        """
        Registers an edge between the caller basic block and callee basic block

        :param from_addr: The address of the basic block that control flow leaves during the transition
        :param to_addr: The address of the basic block that control flow enters during the transition, which is also
                        the address of the target function to call
        :param return_target: The address of instruction to execute after returning from the function. `None` indicates
                            the call does not return.
        :param syscall: Whether this call is a syscall or nor.
        """

        self.blocks.add(from_addr)

        if syscall:
            self._transition_graph.add_edge(from_addr, to_addr, type='syscall')

        else:
            self._transition_graph.add_edge(from_addr, to_addr, type='call')
            if return_target is not None:
                self._transition_graph.add_edge(from_addr, return_target, type='fake_return')

    def return_from_call(self, src_function_addr, to_addr):

        self.blocks.add(to_addr)

        self._transition_graph.add_edge(src_function_addr, to_addr, type='return_from_call')

    def add_block(self, addr):
        '''
        Registers a basic block as part of this function

        @param addr                 The address of the basic block to add
        '''

        self.blocks.add(addr)

        self._transition_graph.add_node(addr)

    def add_return_site(self, return_site_addr):
        '''
        Registers a basic block as a site for control flow to return from this function

        @param return_site_addr     The address of the basic block ending with a return
        '''
        self._ret_sites.add(return_site_addr)

    def add_call_site(self, call_site_addr, call_target_addr, retn_addr):
        '''
        Registers a basic block as calling a function and returning somewhere

        @param call_site_addr       The basic block that ends in a call
        @param call_target_addr     The target of said call
        @param retn_addr            The address that said call will return to
        '''
        self._call_sites[call_site_addr] = (call_target_addr, retn_addr)
        self._retn_addr_to_call_site[retn_addr] = call_site_addr

    def get_call_sites(self):
        '''
        Gets a list of all the basic blocks that end in calls

        @returns                    What the hell do you think?
        '''
        return self._call_sites.keys()

    def get_call_target(self, callsite_addr):
        '''
        Get the target of a call

        @param callsite_addr        The address of the basic block that ends in
                                    a call

        @returns                    The target of said call
        '''
        if callsite_addr in self._call_sites:
            return self._call_sites[callsite_addr][0]
        return None

    def get_call_return(self, callsite_addr):
        '''
        Get the hypothetical return address of a call

        @param callsite_addr        The address of the basic block that ends in
                                    a call

        @returns                    The likely return target of said call
        '''
        if callsite_addr in self._call_sites:
            return self._call_sites[callsite_addr][1]
        return None

    @property
    def basic_blocks(self):
        return self.blocks

    @property
    def transition_graph(self):
        return self._transition_graph

    @property
    def local_transition_graph(self):
        """
        Return a local transition graph that only contain nodes in current function.
        """

        if self._local_transition_graph is not None:
            return self._local_transition_graph

        g = networkx.DiGraph()
        for src, dst, data in self._transition_graph.edges_iter(data=True):
            if src in self.blocks and dst in self.blocks:
                g.add_edge(src, dst, attr_dict=data)
            elif src in self.blocks:
                g.add_node(src)
            elif dst in self.blocks:
                g.add_node(dst)

        for node in self._transition_graph.nodes_iter():
            if node in self.blocks:
                g.add_node(node)

        self._local_transition_graph = g

        return g

    def dbg_print(self):
        '''
        Returns a representation of the list of basic blocks in this function
        '''
        return "[%s]" % (', '.join(('0x%08x' % n) for n in self._transition_graph.nodes()))

    def dbg_draw(self, filename):
        '''
        Draw the graph and save it to a PNG file
        '''
        import matplotlib.pyplot as pyplot # pylint: disable=import-error
        tmp_graph = networkx.DiGraph()
        for edge in self._transition_graph.edges():
            node_a = "0x%08x" % edge[0]
            node_b = "0x%08x" % edge[1]
            if node_b in self._ret_sites:
                node_b += "[Ret]"
            if node_a in self._call_sites:
                node_a += "[Call]"
            tmp_graph.add_edge(node_a, node_b)
        pos = networkx.graphviz_layout(tmp_graph, prog='fdp')
        networkx.draw(tmp_graph, pos, node_size=1200)
        pyplot.savefig(filename)

    def add_argument_register(self, reg_offset):
        '''
        Registers a register offset as being used as an argument to the function

        @param reg_offset           The offset of the register to register
        '''
        if reg_offset in self._function_manager.arg_registers and \
                    reg_offset not in self._argument_registers:
            self._argument_registers.append(reg_offset)

    def add_argument_stack_variable(self, stack_var_offset):
        if stack_var_offset not in self._argument_stack_variables:
            self._argument_stack_variables.append(stack_var_offset)

    @property
    def arguments(self):
        if self.cc is None:
            return self._argument_registers, self._argument_stack_variables
        else:
            return self.cc.arguments

    @property
    def bp_on_stack(self):
        return self._bp_on_stack

    @bp_on_stack.setter
    def bp_on_stack(self, value):
        self._bp_on_stack = value

    @property
    def retaddr_on_stack(self):
        return self._retaddr_on_stack

    @retaddr_on_stack.setter
    def retaddr_on_stack(self, value):
        self._retaddr_on_stack = value

    @property
    def sp_delta(self):
        return self._sp_delta

    @sp_delta.setter
    def sp_delta(self, value):
        self._sp_delta = value

    @property
    def has_return(self):
        return len(self._ret_sites) > 0

class FunctionManager(object):
    '''
    This is a function boundaries management tool. It takes in intermediate
    results during CFG generation, and manages a function map of the binary.
    '''
    def __init__(self, project, cfg):
        self._project = project
        self._cfg = cfg
        # A map that uses function starting address as the key, and maps
        # to a function class
        self._function_map = {}
        self.interfunction_graph = networkx.DiGraph()

        # Registers used for passing arguments around
        self.arg_registers = project.arch.argument_registers

    def _create_function_if_not_exist(self, function_addr):
        if function_addr not in self._function_map:
            self._function_map[function_addr] = Function(self, function_addr)
            self._function_map[function_addr].add_block(function_addr)

    def call_to(self, function_addr, from_addr, to_addr, retn_addr, syscall=False):
        self._create_function_if_not_exist(function_addr)
        self._create_function_if_not_exist(to_addr)
        self._function_map[function_addr].call_to(from_addr, to_addr, retn_addr, syscall=syscall)
        self._function_map[function_addr].add_call_site(from_addr, to_addr, retn_addr)
        self.interfunction_graph.add_edge(function_addr, to_addr)

    def return_from(self, function_addr, from_addr, to_addr=None): #pylint:disable=unused-argument
        self._create_function_if_not_exist(function_addr)
        self._function_map[function_addr].add_return_site(from_addr)

    def transit_to(self, function_addr, from_addr, to_addr):
        self._create_function_if_not_exist(function_addr)
        self._function_map[function_addr].transit_to(from_addr, to_addr)

    def return_from_call(self, function_addr, src_function_addr, to_addr):
        self._create_function_if_not_exist(function_addr)
        self._function_map[function_addr].return_from_call(src_function_addr, to_addr)

    @property
    def functions(self):
        return self._function_map

    def function(self, addr=None, name=None, create_if_not_exist=False):
        if addr:
            if addr in self._function_map:
                return self._function_map[addr]
            elif create_if_not_exist:
                self._create_function_if_not_exist(addr)
                return self._function_map[addr]
        elif name:
            funcs = [ i for i in self._function_map.values() if i.name == name ]
            if funcs:
                return funcs[0]
            else:
                return None
        else:
            return None

    def dbg_print(self):
        result = ''
        for func_addr, func in self._function_map.items():
            f_str = "Function 0x%08x\n%s\n" % (func_addr, func.dbg_print())
            result += f_str
        return result

    def dbg_draw(self):
        for func_addr, func in self._function_map.items():
            filename = "dbg_function_0x%08x.png" % func_addr
            func.dbg_draw(filename)

from .errors import AngrTranslationError



"""
Manage OS-level configuration
"""

import logging
l = logging.getLogger("angr.simos")

from archinfo import ArchARM, ArchMIPS32, ArchX86, ArchAMD64
from simuvex import SimState, SimIRSB, SimStateSystem, SimActionData
from simuvex import s_options as o
from simuvex.s_procedure import SimProcedure, SimProcedureContinuation
from simuvex.s_type import SimTypePointer, SimTypeFunction, SimTypeTop
from cle.metaelf import MetaELF
from cle.backedcgc import BackedCGC


class SimOS(object):
    """A class describing OS/arch-level configuration"""

    def __init__(self, project):
        self.arch = project.arch
        self.proj = project
        self.continue_addr = None

        self.configure_project()

    def configure_project(self):
        """Configure the project to set up global settings (like SimProcedures)"""
        self.continue_addr = self.proj._extern_obj.get_pseudo_addr('angr##simproc_continue')
        self.proj.hook(self.continue_addr, SimProcedureContinuation)

    def state_blank(self, addr=None, initial_prefix=None, **kwargs):
        if kwargs.get('mode', None) is None:
            kwargs['mode'] = self.proj._default_analysis_mode
        if kwargs.get('memory_backer', None) is None:
            kwargs['memory_backer'] = self.proj.loader.memory
        if kwargs.get('arch', None) is None:
            kwargs['arch'] = self.proj.arch

        state = SimState(**kwargs)
        state.regs.sp = self.arch.initial_sp

        if initial_prefix is not None:
            for reg in state.arch.default_symbolic_registers:
                state.registers.store(reg, state.se.Unconstrained(initial_prefix + "_" + reg,
                                                            state.arch.bits,
                                                            explicit_name=True))

        for reg, val, is_addr, mem_region in state.arch.default_register_values:
            if o.ABSTRACT_MEMORY in state.options and is_addr:
                address = state.se.ValueSet(region=mem_region, bits=state.arch.bits, val=val)
                state.registers.store(reg, address)
            else:
                state.registers.store(reg, val)

        if addr is None: addr = self.proj.entry
        state.regs.ip = addr

        state.scratch.ins_addr = addr
        state.scratch.bbl_addr = addr
        state.scratch.stmt_idx = 0
        state.scratch.jumpkind = 'Ijk_Boring'

        state.procedure_data.hook_addr = self.continue_addr
        return state

    def state_entry(self, **kwargs):
        return self.state_blank(**kwargs)

    def state_full_init(self, **kwargs):
        return self.state_entry(**kwargs)

    def prepare_call_state(self, calling_state, initial_state=None,
                           preserve_registers=(), preserve_memory=()):
        '''
        This function prepares a state that is executing a call instruction.
        If given an initial_state, it copies over all of the critical registers to it from the
        calling_state. Otherwise, it prepares the calling_state for action.

        This is mostly used to create minimalistic for CFG generation. Some ABIs, such as MIPS PIE and
        x86 PIE, require certain information to be maintained in certain registers. For example, for
        PIE MIPS, this function transfer t9, gp, and ra to the new state.
        '''

        if isinstance(self.arch, ArchMIPS32):
            if initial_state is not None:
                initial_state = self.state_blank()
            mips_caller_saves = ('s0', 's1', 's2', 's3', 's4', 's5', 's6', 's7', 'gp', 'sp', 'bp', 'ra')
            preserve_registers = preserve_registers + mips_caller_saves + ('t9',)

        if initial_state is None:
            new_state = calling_state.copy()
        else:
            new_state = initial_state.copy()
            for reg in set(preserve_registers):
                new_state.registers.store(reg, calling_state.registers.load(reg))
            for addr, val in set(preserve_memory):
                new_state.memory.store(addr, calling_state.memory.load(addr, val))

        return new_state

class SimLinux(SimOS):
    """OS-specific configuration for *nix-y OSes"""
    def __init__(self, *args, **kwargs):
        super(SimLinux, self).__init__(*args, **kwargs)

        self._loader_addr = None
        self._loader_lock_addr = None
        self._loader_unlock_addr = None
        self._vsyscall_addr = None

    def configure_project(self):
        super(SimLinux, self).configure_project()

        self._loader_addr = self.proj._extern_obj.get_pseudo_addr('angr##loader')
        self._loader_lock_addr = self.proj._extern_obj.get_pseudo_addr('angr##loader_lock')
        self._loader_unlock_addr = self.proj._extern_obj.get_pseudo_addr('angr##loader_unlock')
        self._vsyscall_addr = self.proj._extern_obj.get_pseudo_addr('angr##vsyscall')
        self.proj.hook(self._loader_addr, LinuxLoader, kwargs={'project': self.proj})
        self.proj.hook(self._loader_lock_addr, _dl_rtld_lock_recursive)
        self.proj.hook(self._loader_unlock_addr, _dl_rtld_unlock_recursive)
        self.proj.hook(self._vsyscall_addr, _vsyscall)

        ld_obj = self.proj.loader.linux_loader_object
        if ld_obj is not None:
            tlsfunc = ld_obj.get_symbol('__tls_get_addr')
            if tlsfunc is not None:
                self.proj.hook(tlsfunc.rebased_addr, _tls_get_addr, kwargs={'ld': self.proj.loader})

            _rtld_global = ld_obj.get_symbol('_rtld_global')
            if _rtld_global is not None:
                if isinstance(self.proj.arch, ArchAMD64):
                    self.proj.loader.memory.write_addr_at(_rtld_global.rebased_addr + 0xF08, self._loader_lock_addr)
                    self.proj.loader.memory.write_addr_at(_rtld_global.rebased_addr + 0xF10, self._loader_unlock_addr)

            _rtld_global_ro = ld_obj.get_symbol('_rtld_global_ro')
            if _rtld_global_ro is not None:
                pass

        tls_obj = self.proj.loader.tls_object
        if tls_obj is not None:
            if isinstance(self.proj.arch, ArchAMD64):
                self.proj.loader.memory.write_addr_at(tls_obj.thread_pointer + 0x28, 0x5f43414e4152595f)
                self.proj.loader.memory.write_addr_at(tls_obj.thread_pointer + 0x30, 0x5054524755415244)
            elif isinstance(self.proj.arch, ArchX86):
                self.proj.loader.memory.write_addr_at(tls_obj.thread_pointer + 0x10, self._vsyscall_addr)
            elif isinstance(self.proj.arch, ArchARM):
                self.proj.hook(0xffff0fe0, _kernel_user_helper_get_tls, kwargs={'ld': self.proj.loader})


        # Only set up ifunc resolution if we are using the ELF backend on AMD64
        if isinstance(self.proj.loader.main_bin, MetaELF):
            if isinstance(self.proj.arch, ArchAMD64):
                for binary in self.proj.loader.all_objects:
                    if not isinstance(binary, MetaELF):
                        continue
                    for reloc in binary.relocs:
                        if reloc.symbol is None or reloc.resolvedby is None:
                            continue
                        if reloc.resolvedby.type != 'STT_GNU_IFUNC':
                            continue
                        gotaddr = reloc.addr + binary.rebase_addr
                        gotvalue = self.proj.loader.memory.read_addr_at(gotaddr)
                        if self.proj.is_hooked(gotvalue):
                            continue
                        # Replace it with a ifunc-resolve simprocedure!
                        kwargs = {
                                'proj': self.proj,
                                'funcaddr': gotvalue,
                                'gotaddr': gotaddr,
                                'funcname': reloc.symbol.name
                        }
                        randaddr = self.proj._extern_obj.get_pseudo_addr('ifunc_' + reloc.symbol.name)
                        self.proj.hook(randaddr, IFuncResolver, kwargs=kwargs)
                        self.proj.loader.memory.write_addr_at(gotaddr, randaddr)

    def state_blank(self, fs=None, **kwargs):
        state = super(SimLinux, self).state_blank(**kwargs) #pylint:disable=invalid-name

        if self.proj.loader.tls_object is not None:
            if isinstance(state.arch, ArchAMD64):
                state.regs.fs = self.proj.loader.tls_object.thread_pointer
            elif isinstance(state.arch, ArchX86):
                state.regs.gs = self.proj.loader.tls_object.thread_pointer >> 16
            elif isinstance(state.arch, ArchMIPS32):
                state.regs.ulr = self.proj.loader.tls_object.thread_pointer

        state.register_plugin('posix', SimStateSystem(fs=fs))

        if self.proj.loader.main_bin.is_ppc64_abiv1:
            state.libc.ppc64_abiv = 'ppc64_1'

        return state

    def state_entry(self, args=None, env=None, sargc=None, **kwargs):
        state = super(SimLinux, self).state_entry(**kwargs)

        # Handle default values
        if args is None:
            args = []

        if env is None:
            env = {}

        # Prepare argc
        argc = state.BVV(len(args), state.arch.bits)
        if sargc is not None:
            argc = state.se.Unconstrained("argc", state.arch.bits)

        # Make string table for args/env/auxv
        table = StringTableSpec()

        # Add args to string table
        for arg in args:
            table.add_string(arg)
        table.add_null()

        # Add environment to string table
        for k, v in env.iteritems():
            table.add_string(k + '=' + v)
        table.add_null()

        # Prepare the auxiliary vector and add it to the end of the string table
        # TODO: Actually construct a real auxiliary vector
        aux = []
        for a, b in aux:
            table.add_pointer(a)
            table.add_pointer(b)
        table.add_null()
        table.add_null()

        # Dump the table onto the stack, calculate pointers to args, env, and auxv
        state.memory.store(state.regs.sp, state.BVV(0, 8*16), endness='Iend_BE')
        argv = table.dump(state, state.regs.sp)
        envp = argv + ((len(args) + 1) * state.arch.bytes)
        auxv = argv + ((len(args) + len(env) + 2) * state.arch.bytes)

        # Put argc on stack and fix the stack pointer
        newsp = argv - state.arch.bytes
        state.memory.store(newsp, argc, endness=state.arch.memory_endness)
        state.regs.sp = newsp

        if state.arch.name in ('PPC32',):
            state.stack_push(state.BVV(0, 32))
            state.stack_push(state.BVV(0, 32))
            state.stack_push(state.BVV(0, 32))
            state.stack_push(state.BVV(0, 32))

        # store argc argv envp auxv in the posix plugin
        state.posix.argv = argv
        state.posix.argc = argc
        state.posix.environ = envp
        state.posix.auxv = auxv
        self.set_entry_register_values(state)

        return state

    def set_entry_register_values(self, state):
        for reg, val in state.arch.entry_register_values.iteritems():
            if isinstance(val, (int, long)):
                state.registers.store(reg, val, size=state.arch.bytes)
            elif isinstance(val, (str,)):
                if val == 'argc':
                    state.registers.store(reg, state.posix.argc, size=state.arch.bytes)
                elif val == 'argv':
                    state.registers.store(reg, state.posix.argv)
                elif val == 'envp':
                    state.registers.store(reg, state.posix.environ)
                elif val == 'auxv':
                    state.registers.store(reg, state.posix.auxv)
                elif val == 'ld_destructor':
                    # a pointer to the dynamic linker's destructor routine, to be called at exit
                    # or NULL. We like NULL. It makes things easier.
                    state.registers.store(reg, state.BVV(0, state.arch.bits))
                elif val == 'toc':
                    if self.proj.loader.main_bin.is_ppc64_abiv1:
                        state.registers.store(reg, self.proj.loader.main_bin.ppc64_initial_rtoc)
                else:
                    l.warning('Unknown entry point register value indicator "%s"', val)
            else:
                l.error('What the ass kind of default value is %s?', val)

    def state_full_init(self, **kwargs):
        kwargs['addr'] = self.proj._extern_obj.get_pseudo_addr('angr##loader')
        return super(SimLinux, self).state_full_init(**kwargs)

class SimCGC(SimOS):
    def state_blank(self, fs=None, **kwargs):
        s = super(SimCGC, self).state_blank(**kwargs)  # pylint:disable=invalid-name

        s.register_plugin('posix', SimStateSystem(fs=fs))

        # Create the CGC plugin
        s.get_plugin('cgc')

        # Set CGC-specific options
        #s.options.add(o.CGC_NO_SYMBOLIC_RECEIVE_LENGTH)
        s.options.add(o.CGC_ZERO_FILL_UNCONSTRAINED_MEMORY)

        return s

    def state_entry(self, **kwargs):
        state = super(SimCGC, self).state_entry(**kwargs)

        if isinstance(self.proj.loader.main_bin, BackedCGC):
            for reg, val in self.proj.loader.main_bin.initial_register_values():
                if reg in state.arch.registers:
                    setattr(state.regs, reg, val)
                elif reg == 'eflags':
                    pass
                elif reg == 'fctrl':
                    state.regs.fpround = (val & 0xC00) >> 10
                elif reg == 'fstat':
                    state.regs.fc3210 = (val & 0x4700)
                elif reg == 'ftag':
                    empty_bools = [((val >> (x*2)) & 3) == 3 for x in xrange(8)]
                    tag_chars = [state.BVV(0 if x else 1, 8) for x in empty_bools]
                    for i, tag in enumerate(tag_chars):
                        setattr(state.regs, 'fpu_t%d' % i, tag)
                elif reg in ('fiseg', 'fioff', 'foseg', 'fooff', 'fop'):
                    pass
                elif reg == 'mxcsr':
                    state.regs.sseround = (val & 0x600) >> 9
                else:
                    l.error("What is this register %s I have to translate?", reg)

            # Do all the writes
            writes_backer = self.proj.loader.main_bin.writes_backer
            stdout = 1
            for size in writes_backer:
                if size == 0:
                    continue
                str_to_write = state.posix.files[1].content.load(state.posix.files[1].pos, size)
                a = SimActionData(state, 'file_1_0', 'write', addr=state.BVV(state.posix.files[1].pos, state.arch.bits), data=str_to_write, size=size)
                state.posix.write(stdout, str_to_write, size)
                state.log.add_action(a)

        else:
            # Set CGC-specific variables
            state.regs.eax = 0
            state.regs.ebx = 0
            state.regs.ecx = 0
            state.regs.edx = 0
            state.regs.edi = 0
            state.regs.esi = 0
            state.regs.esp = 0xbaaaaffc
            state.regs.ebp = 0
            #state.regs.eflags = s.BVV(0x202, 32)

            # fpu values
            state.regs.mm0 = 0
            state.regs.mm1 = 0
            state.regs.mm2 = 0
            state.regs.mm3 = 0
            state.regs.mm4 = 0
            state.regs.mm5 = 0
            state.regs.mm6 = 0
            state.regs.mm7 = 0
            state.regs.fpu_tags = 0
            state.regs.fpround = 0
            state.regs.fc3210 = 0x0300
            state.regs.ftop = 0

            # sse values
            state.regs.sseround = 0
            state.regs.xmm0 = 0
            state.regs.xmm1 = 0
            state.regs.xmm2 = 0
            state.regs.xmm3 = 0
            state.regs.xmm4 = 0
            state.regs.xmm5 = 0
            state.regs.xmm6 = 0
            state.regs.xmm7 = 0

        return state

#
# Loader-related simprocedures
#

class IFuncResolver(SimProcedure):
    # pylint: disable=arguments-differ,unused-argument
    def run(self, proj=None, funcaddr=None, gotaddr=None, funcname=None):
        resolve = Callable(proj, funcaddr, SimTypeFunction((), SimTypePointer(self.state.arch, SimTypeTop())))
        try:
            value = resolve()
        except AngrCallableError:
            l.critical("Ifunc failed to resolve!")
            #import IPython; IPython.embed()
            raise
        self.state.memory.store(gotaddr, value, endness=self.state.arch.memory_endness)
        self.add_successor(self.state, value, self.state.se.true, 'Ijk_Boring')

    def __repr__(self):
        return '<IFuncResolver %s>' % self.kwargs.get('funcname', None)

class LinuxLoader(SimProcedure):
    # pylint: disable=unused-argument,arguments-differ,attribute-defined-outside-init
    local_vars = ('initializers',)
    def run(self, project=None):
        self.initializers = project.loader.get_initializers()
        self.run_initializer(project)

    def run_initializer(self, project=None):
        if len(self.initializers) == 0:
            project._simos.set_entry_register_values(self.state)
            self.jump(project.entry)
        else:
            addr = self.initializers.pop(0)
            self.call(addr, (self.state.posix.argc, self.state.posix.argv, self.state.posix.environ), 'run_initializer')

class _tls_get_addr(SimProcedure):
    # pylint: disable=arguments-differ
    def run(self, ptr, ld=None):
        module_id = self.state.memory.load(ptr, self.state.arch.bytes, endness=self.state.arch.memory_endness).model.value
        offset = self.state.memory.load(ptr+self.state.arch.bytes, self.state.arch.bytes, endness=self.state.arch.memory_endness).model.value
        return self.state.BVV(ld.tls_object.get_addr(module_id, offset), self.state.arch.bits)

class _dl_rtld_lock_recursive(SimProcedure):
    # pylint: disable=arguments-differ, unused-argument
    def run(self, lock):
        # For future reference:
        # ++((pthread_mutex_t *)(lock))->__data.__count;
        self.ret()

class _dl_rtld_unlock_recursive(SimProcedure):
    def run(self):
        self.ret()

class _vsyscall(SimProcedure):
    # This is pretty much entirely copied from SimProcedure.ret
    def run(self):
        if self.cleanup:
            self.state.options.discard(o.AST_DEPS)
            self.state.options.discard(o.AUTO_REFS)

        ret_irsb = self.state.arch.disassemble_vex(self.state.arch.ret_instruction, mem_addr=self.addr)
        ret_simirsb = SimIRSB(self.state, ret_irsb, inline=True, addr=self.addr)
        if not ret_simirsb.flat_successors + ret_simirsb.unsat_successors:
            ret_state = ret_simirsb.default_exit
        else:
            ret_state = (ret_simirsb.flat_successors + ret_simirsb.unsat_successors)[0]

        if self.cleanup:
            self.state.options.add(o.AST_DEPS)
            self.state.options.add(o.AUTO_REFS)

        self.add_successor(ret_state, ret_state.scratch.target, ret_state.scratch.guard, 'Ijk_Sys')

class _kernel_user_helper_get_tls(SimProcedure):
    # pylint: disable=arguments-differ
    def run(self, ld=None):
        self.state.regs.r0 = ld.tls_object.thread_pointer
        self.ret()

os_mapping = {
    'unix': SimLinux,
    'unknown': SimOS,
    'windows': SimOS,
    'cgc': SimCGC
}

from .surveyors.caller import Callable
from .errors import AngrCallableError
from .tablespecs import StringTableSpec



class StringSpec(object):
    __immutable = False
    def __init__(self, string=None, sym_length=None, concat=None, name=None, nonnull=False):
        argc = (string is not None) + (sym_length is not None) + (concat is not None)
        if argc == 0 or argc > 1:
            raise ValueError("You must provide one arg!")
        if string is not None:
            self.type = 1
            self._len = len(string)
            self._str = string
        elif sym_length is not None:
            self.type = 2
            self._len = sym_length
            self._name = name if name is not None else ("sym_string_%d" % sym_length)
            self._nonnull = nonnull
        else: # concat is not None
            self._len = 0
            for substring in concat:
                if not isinstance(substring, StringSpec):
                    raise ValueError('All items in concat argument must be StringSpecs!')
                self._len += len(substring)
            self.type = 3
            self._children = concat
        self.__immutable = True

    def dump(self, state, address):
        if self.type == 1:
            for i, c in enumerate(self._str):
                state.memory.store(address + i, state.BVV(ord(c), 8))
        elif self.type == 2:
            state.memory.store(address, state.se.Unconstrained(self._name, 8*self._len))
            if self._nonnull:
                for i in xrange(self._len):
                    state.se.add(state.memory.load(i + address, 1) != state.BVV(0, 8))
        else:
            i = 0
            for child in self._children:
                child.dump(state, address + i)
                i += len(child)

    def __add__(self, other):
        if isinstance(other, str):
            return StringSpec(concat=(self, StringSpec(other)))
        elif isinstance(other, StringSpec):
            return StringSpec(concat=(self, other))
        else:
            return None

    def __radd__(self, other):
        if isinstance(other, str):
            return StringSpec(concat=(StringSpec(other), self))
        elif isinstance(other, StringSpec):
            return StringSpec(concat=(other, self))
        else:
            return None

    def __mul__(self, other):
        return StringSpec(concat=(self)*other)

    def __len__(self):
        return self._len

    def __hash__(self):
        if self.type == 1:
            return hash(self._str) ^ 0xbadfaced
        elif self.type == 2:
            return hash(self._len) ^ hash(self._name) ^ 0xd00dcac3
        elif self.type == 3:
            return hash(tuple(self._children))

    def __setattr__(self, key, value):
        if self.__immutable:
            raise TypeError('Class is immutable')
        else:
            super(StringSpec, self).__setattr__(key, value)


class StringTableSpec:
    def __init__(self):
        self._contents = []
        self._str_len = 0

    def add_string(self, string):
        if isinstance(string, str):
            self._contents.append(StringSpec(string+'\0'))
        elif isinstance(string, StringSpec):
            self._contents.append(string + StringSpec('\0'))
        else:
            raise ValueError('String must be either string literal or StringSpec')
        self._str_len += len(string) + 1

    def add_pointer(self, pointer):
        self._contents.append(pointer)

    def add_null(self):
        self._contents.append(0)

    def dump(self, state, end_addr, align=0x10):
        if isinstance(end_addr, (int, long)):
            end_addr = state.BVV(end_addr, state.arch.bits)
        ptr_size = len(self._contents) * state.arch.bytes
        size = self._str_len + ptr_size
        start_addr = end_addr - size
        zero_fill = (start_addr % align).model.value
        start_addr -= zero_fill
        start_str = start_addr + ptr_size

        ptr_i = start_addr
        str_i = start_str
        for item in self._contents:
            if isinstance(item, StringSpec):
                state.memory.store(ptr_i, str_i, size=state.arch.bytes, endness=state.arch.memory_endness)
                item.dump(state, str_i)
                ptr_i += state.arch.bytes
                str_i += len(item)
            else:
                if isinstance(item, (int, long)):
                    item = state.BVV(item, state.arch.bits)
                state.memory.store(ptr_i, item, size=state.arch.bytes, endness=state.arch.memory_endness)
                ptr_i += state.arch.bytes

        if zero_fill != 0:
            state.memory.store(end_addr - zero_fill, state.BVV(0, 8*zero_fill), endness='Iend_BE')

        return start_addr



from cle.absobj import AbsObj

class AngrExternObject(AbsObj):
    def __init__(self, alloc_size=0x4000):
        super(AngrExternObject, self).__init__('##angr_externs##')
        self._next_addr = 0
        self._lookup_table = {}
        self._alloc_size = alloc_size
        self.memory = 'please never look at this'

    def get_max_addr(self):
        return self._alloc_size + self.rebase_addr

    def get_min_addr(self):
        return self.rebase_addr

    def get_pseudo_addr(self, ident):
        if ident not in self._lookup_table:
            self._lookup_table[ident] = self._next_addr
            self._next_addr += 16
        return self._lookup_table[ident] + self.rebase_addr



from ..errors import AngrMemoryError
from ..analysis import Analysis

from collections import deque
import logging
import math
import networkx
import types

# todo include an explanation of the algorithm
# todo include a method that detects any change other than constants

l = logging.getLogger(name="angr.analyses.bindiff")

# basic block changes
DIFF_TYPE = "type"
DIFF_VALUE = "value"


# exception for trying find basic block changes
class UnmatchedStatementsException(Exception):
    pass


# statement difference classes
class Difference(object):
    def __init__(self, diff_type, value_a, value_b):
        self.type = diff_type
        self.value_a = value_a
        self.value_b = value_b


class ConstantChange(object):
    def __init__(self, offset, value_a, value_b):
        self.offset = offset
        self.value_a = value_a
        self.value_b = value_b


# helper methods
def _euclidean_dist(vector_a, vector_b):
    """
    :param vector_a: list of numbers
    :param vector_b: list of numbers
    :return: the euclidean distance between the two vectors
    """
    dist = 0
    for (x, y) in zip(vector_a, vector_b):
        dist += (x-y)*(x-y)
    return math.sqrt(dist)


def _get_closest_matches(input_attributes, target_attributes):
    """
    :param input_attributes: first dictionary of objects to attribute tuples
    :param target_attributes: second dictionary of blocks to attribute tuples
    :return: dictionary of objects in the input_attributes to the closest objects in the target_attributes
    """
    closest_matches = {}

    # for each object in the first set find the objects with the closest target attributes
    for a in input_attributes:
        best_dist = float('inf')
        best_matches = []
        for b in target_attributes:
            dist = _euclidean_dist(input_attributes[a], target_attributes[b])
            if dist < best_dist:
                best_matches = [b]
                best_dist = dist
            elif dist == best_dist:
                best_matches.append(b)
        closest_matches[a] = best_matches

    return closest_matches


# from http://rosettacode.org/wiki/Levenshtein_distance
def _levenshtein_distance(s1, s2):
    """
    :param s1: A list or string
    :param s2: Another list or string
    :return: The levenshtein distance between the two
    """
    if len(s1) > len(s2):
        s1, s2 = s2, s1
    distances = range(len(s1) + 1)
    for index2, num2 in enumerate(s2):
        new_distances = [index2 + 1]
        for index1, num1 in enumerate(s1):
            if num1 == num2:
                new_distances.append(distances[index1])
            else:
                new_distances.append(1 + min((distances[index1],
                                             distances[index1+1],
                                             new_distances[-1])))
        distances = new_distances
    return distances[-1]


def _normalized_levenshtein_distance(s1, s2, acceptable_differences):
    """
    This function calculates the levenshtein distance but allows for elements in the lists to be different by any number
    in the set acceptable_differences.
    :param s1: A list
    :param s2: Another list
    :param acceptable_differences: A set of numbers. If (s2[i]-s1[i]) is in the set then they are considered equal
    :return:
    """
    if len(s1) > len(s2):
        s1, s2 = s2, s1
        acceptable_differences = set(-i for i in acceptable_differences)
    distances = range(len(s1) + 1)
    for index2, num2 in enumerate(s2):
        new_distances = [index2 + 1]
        for index1, num1 in enumerate(s1):
            if num2 - num1 in acceptable_differences:
                new_distances.append(distances[index1])
            else:
                new_distances.append(1 + min((distances[index1],
                                             distances[index1+1],
                                             new_distances[-1])))
        distances = new_distances
    return distances[-1]


def _is_better_match(x, y, matched_a, matched_b, attributes_dict_a, attributes_dict_b):
    """
    :param x: the first element of a possible match
    :param y: the second element of a possible match
    :param matched_a: the current matches for the first set
    :param matched_b: the current matches for the second set
    :param attributes_dict_a: the attributes for each element in the first set
    :param attributes_dict_b: the attributes for each element in the second set
    :return:
    """
    attributes_x = attributes_dict_a[x]
    attributes_y = attributes_dict_b[y]
    if x in matched_a:
        attributes_match = attributes_dict_b[matched_a[x]]
        if _euclidean_dist(attributes_x, attributes_y) >= _euclidean_dist(attributes_x, attributes_match):
            return False
    if y in matched_b:
        attributes_match = attributes_dict_a[matched_b[y]]
        if _euclidean_dist(attributes_x, attributes_y) >= _euclidean_dist(attributes_y, attributes_match):
            return False
    return True


def differing_constants(block_a, block_b):
    """
    Compares two basic blocks and finds all the constants that differ from the first block to the second
    :param block_a: the first block to compare
    :param block_b: the second block to compare
    :return: returns a list of differing constants in the form of ConstantChange, which has the offset in the block
             and the respective constants.
    """
    if len(block_a.vex.statements) != len(block_b.vex.statements):
        raise UnmatchedStatementsException("Blocks have different numbers of statements")

    start_1 = min(block_a.instruction_addrs)
    start_2 = min(block_b.instruction_addrs)

    changes = []

    # check statements
    current_offset = None
    for statement, statement_2 in zip(block_a.vex.statements + [block_a.vex.next],
                                      block_b.vex.statements + [block_b.vex.next]):
        # sanity check
        if statement.tag != statement_2.tag:
            raise UnmatchedStatementsException("Statement tag has changed")

        if statement.tag == "Ist_IMark":
            if statement.addr - start_1 != statement_2.addr - start_2:
                raise UnmatchedStatementsException("Instruction length has changed")
            current_offset = statement.addr - start_1
            continue

        differences = compare_statement_dict(statement, statement_2)
        for d in differences:
            if d.type != DIFF_VALUE:
                raise UnmatchedStatementsException("Instruction has changed")
            else:
                changes.append(ConstantChange(current_offset, d.value_a, d.value_b))

    return changes


def compare_statement_dict(statement_1, statement_2):
    # should return whether or not the statement's type/effects changed
    # need to return the specific number that changed too
    if type(statement_1) != type(statement_2):
        return [Difference(DIFF_TYPE, None, None)]

    # constants
    if isinstance(statement_1, (int, long, float, str)):
        if statement_1 == statement_2:
            return []
        else:
            return [Difference(None, statement_1, statement_2)]

    # tuples/lists
    if isinstance(statement_1, (tuple, list)):
        if len(statement_1) != len(statement_2):
            return Difference(DIFF_TYPE, None, None)

        differences = []
        for s1, s2 in zip(statement_1, statement_2):
            differences += compare_statement_dict(s1, s2)
        return differences

    # Yan's weird types
    differences = []
    for attr in statement_1.__dict__:
        # don't check arch, property, or methods
        if attr == "arch":
            continue
        if hasattr(statement_1.__class__, attr) and isinstance(getattr(statement_1.__class__, attr), property):
            continue
        if isinstance(getattr(statement_1, attr), types.MethodType):
            continue

        new_diffs = compare_statement_dict(getattr(statement_1, attr), getattr(statement_2, attr))
        # set the difference types
        for diff in new_diffs:
            if diff.type is None:
                diff.type = attr
        differences += new_diffs

    return differences


class FunctionDiff(object):
    """
    This class computes the a diff between two functions
    """
    def __init__(self, function_a, function_b, bindiff=None):
        """
        :param function_a: The first angr Function object to diff
        :param function_b: The second angr Function object
        :param bindiff: An optional Bindiff object. Used for some extra normalization during basic block comparison
        """
        self.function_a = function_a
        self.function_b = function_b
        self._project_a = self.function_a._function_manager._cfg._project
        self._project_b = self.function_b._function_manager._cfg._project
        self._bindiff = bindiff

        self._attributes_a = dict()
        self._attributes_a = dict()

        self._block_matches = set()
        self._unmatched_blocks_from_a = set()
        self._unmatched_blocks_from_b = set()

        self._compute_diff()

    @property
    def probably_identical(self):
        """
        :return: Whether or not these two functions are identical.
        """
        if len(self._unmatched_blocks_from_a | self._unmatched_blocks_from_b) > 0:
            return False
        for (a, b) in self._block_matches:
            if not self.blocks_probably_identical(a, b):
                return False
        return True

    @property
    def identical_blocks(self):
        """
        :return: A list of block matches which appear to be identical
        """
        identical_blocks = []
        for (block_a, block_b) in self._block_matches:
            if self.blocks_probably_identical(block_a, block_b):
                identical_blocks.append((block_a, block_b))
        return identical_blocks

    @property
    def differing_blocks(self):
        """
        :return: A list of block matches which appear to differ
        """
        differing_blocks = []
        for (block_a, block_b) in self._block_matches:
            if not self.blocks_probably_identical(block_a, block_b):
                differing_blocks.append((block_a, block_b))
        return differing_blocks

    @property
    def block_matches(self):
        return self._block_matches

    @property
    def unmatched_blocks(self):
        return self._unmatched_blocks_from_a, self._unmatched_blocks_from_b

    def block_similarity(self, block_a, block_b):
        """
        :param block_a: the first block or address
        :param block_b: the second block or address
        :return: the similarity of the basic blocks, normalized for the base address of the block and function call
        addresses
        """
        if isinstance(block_a, (int, long)):
            try:
                block_a = self._project_a.factory.block(block_a)
            except AngrMemoryError:
                block_a = None
        if isinstance(block_b, (int, long)):
            try:
                block_b = self._project_b.factory.block(block_b)
            except AngrMemoryError:
                block_b = None

        # if both were None then they are assumed to be the same, if only one was the same they are assumed to differ
        if block_a is None and block_b is None:
            return 1.0
        elif block_a is None or block_b is None:
            return 0.0

        # get all elements for computing similarity
        tags_a = [s.tag for s in block_a.vex.statements]
        tags_b = [s.tag for s in block_b.vex.statements]
        consts_a = [c.value for c in block_a.vex.all_constants]
        consts_b = [c.value for c in block_b.vex.all_constants]
        all_registers_a = [s.offset for s in block_a.vex.statements if hasattr(s, "offset")]
        all_registers_b = [s.offset for s in block_b.vex.statements if hasattr(s, "offset")]
        jumpkind_a = block_a.vex.jumpkind
        jumpkind_b = block_b.vex.jumpkind

        # compute total distance
        total_dist = 0
        total_dist += _levenshtein_distance(tags_a, tags_b)
        total_dist += _levenshtein_distance(block_a.vex.operations, block_b.vex.operations)
        total_dist += _levenshtein_distance(all_registers_a, all_registers_b)
        acceptable_differences = self._get_acceptable_constant_differences(block_a, block_b)
        total_dist += _normalized_levenshtein_distance(consts_a, consts_b, acceptable_differences)
        total_dist += 0 if jumpkind_a == jumpkind_b else 1

        # compute similarity
        num_values = max(len(tags_a), len(tags_b))
        num_values += max(len(consts_a), len(consts_b))
        num_values += max(len(block_a.vex.operations), len(block_b.vex.operations))
        num_values += 1  # jumpkind
        similarity = 1 - (float(total_dist) / num_values)

        return similarity

    def blocks_probably_identical(self, block_a, block_b):
        """
        :param block_a: the first block or address
        :param block_b: the second block or address
        :return: Whether or not the blocks appear to be identical
        """
        # handle sim procedure blocks
        if self._project_a.is_hooked(block_a) and self._project_b.is_hooked(block_b):
            return self._project_a._sim_procedures[block_a] == self._project_b._sim_procedures[block_b]

        if isinstance(block_a, (int, long)):
            try:
                block_a = self._project_a.factory.block(block_a)
            except AngrMemoryError:
                block_a = None
        if isinstance(block_b, (int, long)):
            try:
                block_b = self._project_b.factory.block(block_b)
            except AngrMemoryError:
                block_b = None

        # if both were None then they are assumed to be the same, if only one was None they are assumed to differ
        if block_a is None and block_b is None:
            return True
        elif block_a is None or block_b is None:
            return False

        # check differing constants
        try:
            diff_constants = differing_constants(block_a, block_b)
        except UnmatchedStatementsException:
            return False

        # get values of differences that probably indicate no change
        acceptable_differences = self._get_acceptable_constant_differences(block_a, block_b)

        # todo match constants that often move such as those in rodata
        for c in diff_constants:
            if (c.value_a, c.value_b) in self._block_matches:
                # constants point to matched basic blocks
                continue
            if self._bindiff is not None and (c.value_a and c.value_b) in self._bindiff.function_matches:
                # constants point to matched functions
                continue
            # if both are in rodata assume it's good for now
            if ".rodata" in self._project_a.loader.main_bin.sections_map and \
                    ".rodata" in self._project_b.loader.main_bin.sections_map:
                ro_data_a = self._project_a.loader.main_bin.sections_map[".rodata"]
                ro_data_b = self._project_b.loader.main_bin.sections_map[".rodata"]
                base_addr_a = self._project_a.loader.main_bin.rebase_addr
                base_addr_b = self._project_b.loader.main_bin.rebase_addr
                if ro_data_a.contains_addr(c.value_a-base_addr_a) and ro_data_b.contains_addr(c.value_b-base_addr_b):
                    continue
            # if both are in got assume it's good for now
            if ".got.plt" in self._project_a.loader.main_bin.sections_map and \
                    ".got.plt" in self._project_b.loader.main_bin.sections_map:
                ro_data_a = self._project_a.loader.main_bin.sections_map[".got.plt"]
                ro_data_b = self._project_b.loader.main_bin.sections_map[".got.plt"]
                base_addr_a = self._project_a.loader.main_bin.rebase_addr
                base_addr_b = self._project_b.loader.main_bin.rebase_addr
                if ro_data_a.contains_addr(c.value_a-base_addr_a) and ro_data_b.contains_addr(c.value_b-base_addr_b):
                    continue
            # if the difference is equal to the difference in block addr's or successor addr's we'll say it's also okay
            if (c.value_b - c.value_a) in acceptable_differences:
                continue
            # otherwise they probably are different
            return False

        # the blocks appear to be identical
        return True

    @staticmethod
    def _compute_block_attributes(function):
        """
        :param function: An angr function object
        :return: a dictionary of basic block addresses to tuples of attributes
        """
        # The attributes we use are the distance form function start, distance from function exit and whether
        # or not it has a subfunction call
        distances_from_start = FunctionDiff._distances_from_function_start(function)
        distances_from_exit = FunctionDiff._distances_from_function_exit(function)
        call_sites = function.get_call_sites()

        attributes = {}
        for block in function.local_transition_graph.nodes():
            number_of_subfunction_calls = 1 if block in call_sites else 0
            # there really shouldn't be blocks that can't be reached from the start, but there are for now
            dist_start = distances_from_start[block] if block in distances_from_start else 10000
            dist_exit = distances_from_exit[block] if block in distances_from_exit else 10000
            attributes[block] = (dist_start, dist_exit, number_of_subfunction_calls)

        return attributes

    @staticmethod
    def _distances_from_function_start(function):
        """
        :param function: An angr Function object
        :return: a dictionary of basic block addresses and their distance to the start of the function
        """
        return networkx.single_source_shortest_path_length(function.local_transition_graph, function.startpoint)

    @staticmethod
    def _distances_from_function_exit(function):
        """
        :param function: An angr Function object
        :return: a dictionary of basic block addresses and their distance to the exit of the function
        """
        reverse_graph = function.local_transition_graph.reverse()
        # we aren't guaranteed to have an exit from the function so explicitly add the node
        reverse_graph.add_node("start")
        for n in function.local_transition_graph.nodes():
            if len(function.local_transition_graph.successors(n)) == 0:
                reverse_graph.add_edge("start", n)

        dists = networkx.single_source_shortest_path_length(reverse_graph, "start")

        # remove temp node
        del dists["start"]

        # correct for the added node
        for n in dists:
            dists[n] -= 1

        return dists

    def _compute_diff(self):
        """
        Computes the diff of the functions and saves the result
        """
        # get the attributes for all blocks
        l.debug("Computing diff of functions: %s, %s", hex(self.function_a.startpoint), hex(self.function_b.startpoint))
        self.attributes_a = self._compute_block_attributes(self.function_a)
        self.attributes_b = self._compute_block_attributes(self.function_b)

        # get the initial matches
        initial_matches = self._get_block_matches(self.attributes_a, self.attributes_b,
                                                  tiebreak_with_block_similarity=False)

        # Use a queue so we process matches in the order that they are found
        to_process = deque(initial_matches)

        # Keep track of which matches we've already added to the queue
        processed_matches = set((x, y) for (x, y) in initial_matches)

        # Keep a dict of current matches, which will be updated if better matches are found
        matched_a = dict()
        matched_b = dict()
        for (x, y) in processed_matches:
            matched_a[x] = y
            matched_b[y] = x

        # while queue is not empty
        while to_process:
            (block_a, block_b) = to_process.pop()

            # we could find new matches in the successors or predecessors of functions
            block_a_succ = self.function_a.local_transition_graph.successors(block_a)
            block_b_succ = self.function_b.local_transition_graph.successors(block_b)
            block_a_pred = self.function_a.local_transition_graph.predecessors(block_a)
            block_b_pred = self.function_b.local_transition_graph.predecessors(block_b)

            # propagate the difference in blocks as delta
            delta = tuple((i-j) for i, j in zip(self.attributes_b[block_b], self.attributes_a[block_a]))

            # get possible new matches
            new_matches = self._get_block_matches(self.attributes_a, self.attributes_b, block_a_succ, block_b_succ,
                                                  delta, tiebreak_with_block_similarity=True)
            new_matches += self._get_block_matches(self.attributes_a, self.attributes_b, block_a_pred, block_b_pred,
                                                   delta, tiebreak_with_block_similarity=True)

            # for each of the possible new matches add it if it improves the matching
            for (x, y) in new_matches:
                if (x, y) not in processed_matches:
                    processed_matches.add((x, y))
                    # if it's a better match than what we already have use it
                    if _is_better_match(x, y, matched_a, matched_b, self.attributes_a, self.attributes_b):
                        if x in matched_a:
                            old_match = matched_a[x]
                            del matched_b[old_match]
                        if y in matched_b:
                            old_match = matched_b[y]
                            del matched_a[old_match]
                        matched_a[x] = y
                        matched_b[y] = x
                        to_process.appendleft((x, y))

        # reformat matches into a set of pairs
        self._block_matches = set((x, y) for (x, y) in matched_a.items())

        # get the unmatched blocks
        self._unmatched_blocks_from_a = set(x for x in self.function_a.local_transition_graph.nodes() if x not in matched_a)
        self._unmatched_blocks_from_b = set(x for x in self.function_b.local_transition_graph.nodes() if x not in matched_b)

    def _get_block_matches(self, attributes_a, attributes_b, filter_set_a=None, filter_set_b=None, delta=(0, 0, 0),
                           tiebreak_with_block_similarity = False):
        """
        :param attributes_a: dict of blocks to their attributes
        :param attributes_b: dict of blocks to their attributes
        :param filter_set_a: an optional set to limit attributes_a to the blocks in this set
        :param filter_set_b: an optional set to limit attributes_b to the blocks in this set
        :param delta: offset to add to each vector in attributes_a
        :return: a list of tuples of matching objects
        """
        # get the attributes that are in the sets
        if filter_set_a is None:
            filtered_attributes_a = {k: v for k, v in attributes_a.items()}
        else:
            filtered_attributes_a = {k: v for k, v in attributes_a.items() if k in filter_set_a}

        if filter_set_b is None:
            filtered_attributes_b = {k: v for k, v in attributes_b.items()}
        else:
            filtered_attributes_b = {k: v for k, v in attributes_b.items() if k in filter_set_b}

        # add delta
        for k in filtered_attributes_a:
            filtered_attributes_a[k] = tuple((i+j) for i, j in zip(filtered_attributes_a[k], delta))
        for k in filtered_attributes_b:
            filtered_attributes_b[k] = tuple((i+j) for i, j in zip(filtered_attributes_b[k], delta))

        # get closest
        closest_a = _get_closest_matches(filtered_attributes_a, filtered_attributes_b)
        closest_b = _get_closest_matches(filtered_attributes_b, filtered_attributes_a)

        if tiebreak_with_block_similarity:
            # use block similarity to break ties in the first set
            for a in closest_a:
                if len(closest_a[a]) > 1:
                    best_similarity = 0
                    best = []
                    for x in closest_a[a]:
                        block_a = self._project_a.factory.block(a)
                        block_b = self._project_b.factory.block(x)
                        similarity = self.block_similarity(block_a, block_b)
                        if similarity > best_similarity:
                            best_similarity = similarity
                            best = [x]
                        elif similarity == best_similarity:
                            best.append(x)
                    closest_a[a] = best

            # use block similarity to break ties in the second set
            for b in closest_b:
                if len(closest_b[b]) > 1:
                    best_similarity = 0
                    best = []
                    for x in closest_b[b]:
                        block_a = self._project_a.factory.block(x)
                        block_b = self._project_b.factory.block(b)
                        similarity = self.block_similarity(block_a, block_b)
                        if similarity > best_similarity:
                            best_similarity = similarity
                            best = [x]
                        elif similarity == best_similarity:
                            best.append(x)
                    closest_b[b] = best

        # a match (x,y) is good if x is the closest to y and y is the closest to x
        matches = []
        for a in closest_a:
            if len(closest_a[a]) == 1:
                match = closest_a[a][0]
                if len(closest_b[match]) == 1 and closest_b[match][0] == a:
                    matches.append((a, match))

        return matches

    def _get_acceptable_constant_differences(self, block_a, block_b):
        jumpkind_a = block_a.vex.jumpkind
        jumpkind_b = block_b.vex.jumpkind

        # keep a set of the acceptable differences in constants between the two blocks
        acceptable_differences = set()
        acceptable_differences.add(0)

        block_a_base = block_a.instruction_addrs[0]
        block_b_base = block_b.instruction_addrs[0]
        acceptable_differences.add(block_b_base - block_a_base)

        # get matching successors
        if jumpkind_a == 'Ijk_Call' and jumpkind_b == 'Ijk_Call':
            block_a_call = self.function_a.get_call_target(block_a_base)
            block_b_call = self.function_b.get_call_target(block_b_base)
            # these can be none if we couldn't resolve the call target
            if block_a_call is not None and block_b_call is not None:
                acceptable_differences.add(block_b_call - block_a_call)
                acceptable_differences.add((block_b_call - block_b_base) - (block_a_call - block_a_base))

        # get the difference between the data segments
        # this is hackish
        if ".bss" in self._project_a.loader.main_bin.sections_map and \
                ".bss" in self._project_b.loader.main_bin.sections_map:
            bss_a = self._project_a.loader.main_bin.sections_map[".bss"].min_addr
            bss_b = self._project_b.loader.main_bin.sections_map[".bss"].min_addr
            acceptable_differences.add(bss_b - bss_a)
            acceptable_differences.add((bss_b - block_b_base) - (bss_a - block_a_base))

        return acceptable_differences


class BinDiff(Analysis):
    """
    This class computes the a diff between two binaries represented by angr Projects
    """
    def __init__(self, other_project):
        """
        :param other_project: The second project to diff
        """
        self.cfg_a = self._p.analyses.CFG(context_sensitivity_level=1,
                                          keep_input_state=True,
                                          enable_symbolic_back_traversal=True)
        self.cfg_b = other_project.analyses.CFG(context_sensitivity_level=1,
                                                keep_input_state=True,
                                                enable_symbolic_back_traversal=True)

        self._attributes_a = dict()
        self._attributes_a = dict()

        self._function_diffs = dict()
        self.function_matches = set()
        self._unmatched_functions_from_a = set()
        self._unmatched_functions_from_b = set()

        self._compute_diff()

    def functions_probably_identical(self, func_a_addr, func_b_addr):
        """
        :param func_a_addr: The address of the first function (in the first binary)
        :param func_b_addr: The address of the second function (in the second binary)
        :return: whether or not the functions appear to be identical
        """
        if self.cfg_a._project.is_hooked(func_a_addr) and self.cfg_b._project.is_hooked(func_b_addr):
            return self.cfg_a._project._sim_procedures[func_a_addr] == self.cfg_b._project._sim_procedures[func_b_addr]

        func_diff = self.get_function_diff(func_a_addr, func_b_addr)
        return func_diff.probably_identical

    @property
    def identical_functions(self):
        """
        :return: A list of function matches that appear to be identical
        """
        identical_funcs = []
        for (func_a, func_b) in self.function_matches:
            if self.functions_probably_identical(func_a, func_b):
                identical_funcs.append((func_a, func_b))
        return identical_funcs

    @property
    def differing_functions(self):
        """
        :return: A list of function matches that appear to differ
        """
        different_funcs = []
        for (func_a, func_b) in self.function_matches:
            if not self.functions_probably_identical(func_a, func_b):
                different_funcs.append((func_a, func_b))
        return different_funcs

    @property
    def unmatched_functions(self):
        return self._unmatched_functions_from_a, self._unmatched_functions_from_b

    # gets the diff of two functions in the binaries
    def get_function_diff(self, function_addr_a, function_addr_b):
        """
        :param function_addr_a: The address of the first function (in the first binary)
        :param function_addr_b: The address of the second function (in the second binary)
        :return: the FunctionDiff of the two functions
        """
        pair = (function_addr_a, function_addr_b)
        if pair not in self._function_diffs:
            function_a = self.cfg_a.function_manager.function(function_addr_a)
            function_b = self.cfg_b.function_manager.function(function_addr_b)
            self._function_diffs[pair] = FunctionDiff(function_a, function_b, self)
        return self._function_diffs[pair]

    @staticmethod
    def _compute_function_attributes(cfg):
        """
        :param cfg: An angr CFG object
        :return: a dictionary of function addresses to tuples of attributes
        """
        # the attributes we use are the number of basic blocks, number of edges, and number of subfunction calls
        attributes = dict()
        for function_addr in cfg.function_manager.functions:
            function = cfg.function_manager.function(function_addr)
            number_of_basic_blocks = len(function.local_transition_graph.nodes())
            number_of_edges = len(function.local_transition_graph.edges())
            number_of_subfunction_calls = len(cfg.function_manager.interfunction_graph.successors(function_addr))
            attributes[function_addr] = (number_of_basic_blocks, number_of_edges, number_of_subfunction_calls)

        return attributes

    def _compute_diff(self):
        # get the attributes for all functions
        self.attributes_a = self._compute_function_attributes(self.cfg_a)
        self.attributes_b = self._compute_function_attributes(self.cfg_b)

        # get the initial matches
        initial_matches = self._get_function_matches(self.attributes_a, self.attributes_b)

        # Use a queue so we process matches in the order that they are found
        to_process = deque(initial_matches)

        # Keep track of which matches we've already added to the queue
        processed_matches = set((x, y) for (x, y) in initial_matches)

        # Keep a dict of current matches, which will be updated if better matches are found
        matched_a = dict()
        matched_b = dict()
        for (x, y) in processed_matches:
            matched_a[x] = y
            matched_b[y] = x

        # while queue is not empty
        while to_process:
            (func_a, func_b) = to_process.pop()

            # we could find new matches in the successors or predecessors of functions
            func_a_succ = self.cfg_a.function_manager.interfunction_graph.successors(func_a)
            func_b_succ = self.cfg_b.function_manager.interfunction_graph.successors(func_b)
            func_a_pred = self.cfg_a.function_manager.interfunction_graph.predecessors(func_a)
            func_b_pred = self.cfg_b.function_manager.interfunction_graph.predecessors(func_b)

            # get possible new matches
            new_matches = set(self._get_function_matches(self.attributes_a, self.attributes_b,
                                                         func_a_succ, func_b_succ))
            new_matches |= set(self._get_function_matches(self.attributes_a, self.attributes_b,
                                                          func_a_pred, func_b_pred))

            # could also find matches as function calls of matched basic blocks
            function_a = self.cfg_a.function_manager.function(func_a)
            function_b = self.cfg_b.function_manager.function(func_b)
            basic_block_matches = self.get_function_diff(func_a, func_b).block_matches
            for (a, b) in basic_block_matches:
                if a in function_a.get_call_sites() and b in function_b.get_call_sites():
                    new_matches.add((function_a.get_call_target(a), function_b.get_call_target(b)))

            # for each of the possible new matches add it if it improves the matching
            for (x, y) in new_matches:
                if (x, y) not in processed_matches:
                    processed_matches.add((x, y))
                    # if it's a better match than what we already have use it
                    l.debug("Checking function match %s, %s", hex(x), hex(y))
                    if _is_better_match(x, y, matched_a, matched_b, self.attributes_a, self.attributes_b):
                        l.debug("Adding match %s, %s", hex(x), hex(y))
                        if x in matched_a:
                            old_match = matched_a[x]
                            del matched_b[old_match]
                        if y in matched_b:
                            old_match = matched_b[y]
                            del matched_a[old_match]
                        matched_a[x] = y
                        matched_b[y] = x
                        to_process.appendleft((x, y))

        # reformat matches into a set of pairs
        self.function_matches = set((x, y) for (x, y) in matched_a.items())

        # get the unmatched blocks
        self._unmatched_functions_from_a = set(x for x in self.cfg_a.function_manager.functions if x not in matched_a)
        self._unmatched_functions_from_b = set(x for x in self.cfg_b.function_manager.functions if x not in matched_b)

        # remove unneeded function diffs
        for (x, y) in dict(self._function_diffs):
            if (x, y) not in self.function_matches:
                del self._function_diffs[(x, y)]

    @staticmethod
    def _get_function_matches(attributes_a, attributes_b, filter_set_a=None, filter_set_b=None):
        """
        :param attributes_a: dict of functions to their attributes
        :param attributes_b: dict of functions to their attributes
        :param filter_set_a: an optional set to limit attributes_a to the functions in this set
        :param filter_set_b: an optional set to limit attributes_b to the functions in this set
        :return: a list of tuples of matching objects
        """
        # get the attributes that are in the sets
        if filter_set_a is None:
            filtered_attributes_a = {k: v for k, v in attributes_a.items()}
        else:
            filtered_attributes_a = {k: v for k, v in attributes_a.items() if k in filter_set_a}

        if filter_set_b is None:
            filtered_attributes_b = {k: v for k, v in attributes_b.items()}
        else:
            filtered_attributes_b = {k: v for k, v in attributes_b.items() if k in filter_set_b}

        # get closest
        closest_a = _get_closest_matches(filtered_attributes_a, filtered_attributes_b)
        closest_b = _get_closest_matches(filtered_attributes_b, filtered_attributes_a)

        # a match (x,y) is good if x is the closest to y and y is the closest to x
        matches = []
        for a in closest_a:
            if len(closest_a[a]) == 1:
                match = closest_a[a][0]
                if len(closest_b[match]) == 1 and closest_b[match][0] == a:
                    matches.append((a, match))

        return matches



from collections import defaultdict
import logging

import networkx

import simuvex

from .pathprioritizer import PathPrioritizer
from .errors import AngrAnnotatedCFGError
from .analyses.cfg import CFGNode

l = logging.getLogger("angr.annocfg")

class AnnotatedCFG(object):
    # cfg : class CFG
    def __init__(self, project, cfg=None, target_irsb_addr=None, detect_loops=False):
        self._project = project

        self._cfg = None
        self._target = None

        self._run_statement_whitelist = defaultdict(list)
        self._exit_taken = defaultdict(list)
        self._addr_to_run = {}
        self._addr_to_last_stmt_id = {}
        self._loops = []
        self._path_merge_points = [ ]
        self._path_prioritizer = None


        if cfg is not None:
            self._cfg = cfg

            if target_irsb_addr is not None:
                self._target = self._cfg.get_any_node(target_irsb_addr)
                self._path_prioritizer = PathPrioritizer(self._cfg, self._target)

        # if detect_loops:
        #     self._detect_loops()

        if self._cfg is not None:
            for run in self._cfg.nodes():
                self._addr_to_run[self.get_addr(run)] = run

    def __getstate__(self):
        state = { }
        state['_run_statement_whitelist'] = self._run_statement_whitelist
        state['_exit_taken'] = self._exit_taken
        # state['_addr_to_run'] = self._addr_to_run
        state['_addr_to_last_stmt_id'] = self._addr_to_last_stmt_id
        state['_loops'] = self._loops
        state['_path_merge_points'] = self._path_merge_points
        state['_path_prioritizer'] = self._path_prioritizer
        state['_cfg'] = None
        state['_project'] = None
        state['_addr_to_run'] = None
        return state

    def _detect_loops(self):
        temp_graph = networkx.DiGraph()
        for source, target_list in self._cfg._edge_map.items():
            for target in target_list:
                temp_graph.add_edge(source, target)
        ctr = 0
        for loop_lst in networkx.simple_cycles(temp_graph):
            l.debug("A loop is found. %d", ctr)
            ctr += 1
            loop = (tuple([x[-1] for x in loop_lst]))
            print " => ".join(["0x%08x" % x for x in loop])
            self.add_loop(loop)

    def from_digraph(self, digraph):
        '''
        Initialize this AnnotatedCFG object with a networkx.DiGraph consisting of the following
        form of nodes:

        Tuples like (SimRun address, statement ID)

        Those nodes are connected by edges indicating the execution flow.

        :param digraph: A networkx.DiGraph object
        :return: None
        '''

        for n1, n2 in digraph.edges():
            addr1, stmt_idx1 = n1
            addr2, stmt_idx2 = n2

            if addr1 != addr2:
                # There is a control flow transition from SimRun `addr1` to SimRun `addr2`
                self.add_exit_to_whitelist(addr1, addr2)

            self.add_statements_to_whitelist(addr1, (stmt_idx1,))
            self.add_statements_to_whitelist(addr2, (stmt_idx2,))

    def get_addr(self, run):
        if isinstance(run, simuvex.SimIRSB):
            return run.first_imark.addr
        elif isinstance(run, simuvex.SimProcedure):
            # pseudo_addr = self._project.get_pseudo_addr_for_sim_procedure(run)
            pseudo_addr = run.addr
            return pseudo_addr
        elif isinstance(run, CFGNode):
            return run.addr
        elif type(run) in (int, long):
            return run
        else:
            raise AngrAnnotatedCFGError("Unknown type '%s' of the 'run' argument" % type(run))

    def add_simrun_to_whitelist(self, simrun):
        addr = self.get_addr(simrun)
        self._run_statement_whitelist[addr] = True

    def add_statements_to_whitelist(self, simrun, stmt_ids):
        addr = self.get_addr(simrun)
        if type(stmt_ids) is bool:
            if type(self._run_statement_whitelist[addr]) is list and self._run_statement_whitelist[addr]:
                raise Exception("WTF")
            self._run_statement_whitelist[addr] = stmt_ids
        else:
            self._run_statement_whitelist[addr].extend(stmt_ids)
            self._run_statement_whitelist[addr] = \
                sorted(list(set(self._run_statement_whitelist[addr])))

    def add_exit_to_whitelist(self, run_from, run_to):
        addr_from = self.get_addr(run_from)
        addr_to = self.get_addr(run_to)
        self._exit_taken[addr_from].append(addr_to)

    def set_last_stmt(self, run, stmt_id):
        addr = self.get_addr(run)
        self._addr_to_last_stmt_id[addr] = stmt_id

    def add_loop(self, loop_tuple):
        '''
        A loop tuple contains a series of IRSB addresses that form a loop. Ideally
        it always starts with the first IRSB that we meet during the execution.
        '''
        self._loops.append(loop_tuple)

    def set_path_merge_points(self, points):
        self._path_merge_points = points.copy()

    def should_take_exit(self, addr_from, addr_to):
        if addr_from in self._exit_taken:
            return addr_to in self._exit_taken[addr_from]

        return False

    def should_execute_statement(self, addr, stmt_id):
        if self._run_statement_whitelist is None:
            return True
        elif addr in self._run_statement_whitelist:
            return stmt_id in self._run_statement_whitelist[addr]
        return False

    def get_run(self, addr):
        if addr in self._addr_to_run:
            return self._addr_to_run[addr]
        return None

    def get_whitelisted_statements(self, addr):
        '''
        @return: True if all statements are whitelisted
        '''
        if addr in self._run_statement_whitelist:
            if self._run_statement_whitelist[addr] is True:
                return None # This is the default value used in SimuVEX to say
                            # we execute all statements in this basic block. A
                            # little weird...
            else:
                return self._run_statement_whitelist[addr]
        return []

    def get_last_statement_index(self, addr):
        if addr in self._addr_to_last_stmt_id:
            return self._addr_to_last_stmt_id[addr]
        return None

    def get_loops(self):
        return self._loops

    def debug_print(self):
        l.debug("SimRuns:")
        for addr, run in self._addr_to_run.items():
            if addr is None:
                continue
            l.debug("0x%08x => %s", addr, run)
        l.debug("statements: ")
        for addr, stmts in self._run_statement_whitelist.items():
            if addr is None:
                continue
            l.debug("Address 0x%08x:", addr)
            l.debug(stmts)
        l.debug("Loops: ")
        for loop in self._loops:
            s = ""
            for addr in loop:
                s += "0x%08x -> " % addr
            l.debug(s)
        l.debug("=== EOF ===")

    def _dbg_print_irsb(self, irsb_addr, project=None):
        '''
        Only for debugging purposes.
        '''
        if self._addr_to_run is None:
            if project is None:
                raise Exception("Dict addr_to_run is empty. " + \
                                "Give me a project, and I'll recreate the IRSBs for you.")
            else:
                irsb = project.factory.sim_run(project.exit_to(irsb_addr, project.initial_state(mode="static")))

        elif irsb_addr not in self._addr_to_run:
            raise Exception("0x%08x not found." % irsb_addr)
        else:
            irsb = self._addr_to_run[irsb_addr]
            if not isinstance(irsb, simuvex.SimIRSB):
                raise Exception("0x%08x is not a SimIRSB instance.")
        statements = irsb.statements
        whitelist = self.get_whitelisted_statements(irsb_addr)
        for i in range(0, len(statements)):
            if whitelist is True or i in whitelist:
                line = "+"
            else:
                line = "-"
            line += "[% 3d] " % i
            # We cannot get data returned by pp(). WTF?
            print line,
            statements[i].stmt.pp()
            print ""

    def keep_path(self, path):
        '''
        Given a path, returns True if the path should be kept, False if it should be cut.
        '''
        if len(path.addr_backtrace) < 2:
            return True

        return self.should_take_exit(path.addr_backtrace[-2], path.addr_backtrace[-1])

    def filter_path(self, path):
        '''
        Only for debugging purposes!
        '''
        #s = [4286871328L, 4286871380L, 4286871392L, 4288524088, 4286871416L, 4286871444L, 4288360060, 4286871460L, 4286871468L, 4286871476L, 4288441092, 4286871484L, 4286871496L, 4286871528L, 4286871536L, 4286871568L, 4286871576L, 4286871596L, 4286871604L, 4286871624L, 4286871632L, 4286871640L, 4286871652L, 4286871664L, 4286872764, 4286871684L, 4286871708L, 4286871788L, 4286871800L, 4286871816L, 4288504180L, 4286871820L, 4286871840L, 4288504180L, 4286871844L, 4286871880L, 4286871896L, 4286871400L, 4288524088, 4286871416L, 4286871444L, 4288360060, 4286871460L, 4286871468L, 4286871476L, 4288441092, 4286871484L, 4286871496L, 4286871528L, 4286871536L, 4286871568L, 4286871576L, 4286871596L, 4286871604L, 4286871624L, 4286871632L, 4286871640L, 4286871652L, 4286871664L, 4286872764, 4286871684L, 4286871708L, 4286871788L, 4286871800L, 4286871816L, 4288504180L, 4286871820L, 4286871840L, 4288504180L, 4286871844L, 4286871880L, 4286871896L, 4286871400L, 4288524088, 4286871416L, 4286871444L, 4288360060, 4286871460L, 4286871468L, 4286871476L, 4288441092, 4286871484L, 4286871496L, 4286871528L, 4286871536L, 4286871568L, 4286871576L, 4286871596L, 4286871604L, 4286871624L, 4286871632L, 4286871640L, 4286871652L, 4286871664L, 4286872764, 4286871684L, 4286871708L, 4286871788L, 4286871800L, 4286871816L, 4288504180L, 4286871820L, 4286871840L, 4288504180L, 4286871844L, 4286871880L, 4286871896L, 4286871400L, 4288524088, 4286871416L, 4286871444L, 4288360060, 4286871460L, 4286871468L, 4286871476L, 4288441092, 4286871484L, 4286871496L, 4286871528L, 4286871536L, 4286871568L, 4286871576L, 4286871596L, 4286871604L, 4286871624L, 4286871632L, 4286871640L, 4286871652L, 4286871664L, 4286872764, 4286871684L, 4286871708L, 4286871788L, 4286871800L, 4286871816L, 4288504180L, 4286871820L, 4286871840L, 4288504180L, 4286871844L, 4286871880L, 4286871896L, 4286871400L, 4288524088, 4286871416L, 4286871444L, 4288360060, 4286871460L, 4286871468L, 4286871476L, 4288441092, 4286871484L, 4286871496L, 4286871528L, 4286871536L, 4286871568L, 4286871576L, 4286871596L, 4286871604L, 4286871624L, 4286871632L, 4286871640L, 4286871652L, 4286871664L, 4286872764, 4286871684L, 4286871708L, 4286871788L, 4286871800L, 4286871816L, 4288504180L, 4286871820L, 4286871840L, 4288504180L, 4286871844L, 4286871880L, 4286871896L, 4286871400L, 4288524088, 4286871416L, 4286871444L, 4288360060, 4286871460L, 4286871468L, 4286871476L, 4288441092, 4286871484L, 4286871496L, 4286871528L, 4286871536L, 4286871568L, 4286871576L, 4286871596L, 4286871604L, 4286871624L, 4286871632L, 4286871640L, 4286871652L, 4286871664L, 4286872764, 4286871684L, 4286871708L, 4286871788L, 4286871800L, 4286871816L, 4288504180L, 4286871820L, 4286871840L, 4288504180L, 4286871844L, 4286871880L, 4286871896L, 4286871400L, 4288524088, 4286871416L, 4286871444L, 4288360060, 4286871460L, 4286871468L, 4286871476L, 4288441092, 4286871484L, 4286871496L, 4286871528L, 4286871536L, 4286871568L, 4286871576L, 4286871596L, 4286871604L, 4286871624L, 4286871632L, 4286871640L, 4286871652L, 4286871664L, 4286872764, 4286871684L, 4286871708L, 4286871788L, 4286871800L, 4286871816L, 4288504180L, 4286871820L, 4286871840L, 4288504180L, 4286871844L, 4286871880L, 4286871896L, 4286871400L, 4288524088, 4286871416L, 4286871444L, 4288360060, 4286871460L, 4286871468L, 4286871476L, 4288441092, 4286871484L, 4286871496L, 4286871528L, 4286871544L, 4286871596L, 4286871604L, 4286871624L, 4286871688L, 4286872764, 4286871708L, 4286871720L, 4286871764L, 4286871992L]
        #result = not any(i != j for i,j in zip(path.addr_backtrace, s))
        #if result:
        #    if not hasattr(self, "_fuckyou"):
        #        self._fuckyou = True
        #    if self._fuckyou and 0xff8479e0 in path.addr_backtrace:
        #        import ipdb
        #        ipdb.set_trace()
        #    return True
        #else:
        #    return False
        ############### INSANE
        return True

        if not hasattr(self, "_fuckyou"):
            self._fuckyou = True
        if self._fuckyou and 0xff8479e0 in path.addr_backtrace:
            import ipdb
            ipdb.set_trace()

        telnet = 0xff8479b8
        input_char = 0xff847818
        loop_head = 0xff847768
        path = path.addr_backtrace
        if telnet in path:
            if path.count(telnet) >= 3:
                return False
            pos1 = path.index(telnet)
            if path[: pos1].count(input_char) < 7:
                return False
            if pos1 + 1 < len(path):
                new_path = path[pos1 + 1 : ]
                if telnet in new_path:
                    pos2 = new_path.index(telnet)
                    new_path = new_path[ : pos2]
                    if new_path.count(input_char) == 0:
                        return False
                    if new_path.count(input_char) > 3:
                        return False
            return True
        else:
            if path.count(loop_head) > 8:
                return False
            else:
                return True


    def merge_points(self, path):
        addr = path.addr
        if addr in self._path_merge_points:
            return {self._path_merge_points[addr]}
        else:
            return set()

    def path_priority(self, path):
        '''
        Given a path, returns the path priority. A lower number means a higher priority.
        '''
        return self._path_prioritizer.get_priority(path)
        # FIXME: Testing code for smartmeter stuff!
        # bt = ",".join(path.backtrace)
        # if "ff8479b8" in bt:
        #     if bt.count("ff8479b8") >= 3:
        #         # l.debug("I saw... we shouldn't get into login function too many times...")
        #         return 200
        #     else:
        #         prior = bt[ : bt.find("ff8479b8")]
        #         if prior.count("ff847818") < 4:
        #             l.debug("Entered too early...")
        #             return 200
        #         if bt.count("ff8479f4") == 1:
        #             # l.debug("I saw path going to password_asking phase!")
        #             return 1
        #         l.debug("I saw path going to checking phase!")
        #         return 2
        # else:
        #     if bt.count("ff847818") < 7 and bt.count("ff847818") >= 4:
        #         # l.debug("I saw monkey-input process...")
        #         return 3
        #     elif bt.count("ff847818") < 4:
        #         return 50
        # return 100



import ana
import simuvex
import claripy
import mulpyplexer

import logging
l = logging.getLogger('angr.path_group')


class CallReturn(simuvex.SimProcedure):
    NO_RET = True

    def run(self):
        l.info("A PathGroup.call-created path returned!")
        return


class PathGroup(ana.Storable):
    '''
    Path groups are the future.

    Path groups allow you to wrangle multiple paths in a slick way.  Paths are
    organized into "stashes", which you can step forward, filter, merge, and
    move around as you wish. This allows you to, for example, step two
    different stashes of paths at different rates, then merge them together.

    Note that path groups are immutable by default (all operations will return
    new PathGroup objects). See the immutable argument to __init__.

    Stashes can be accessed as attributes (i.e., pg.active). A mulpyplexed
    stash can be retrieved by prepending the name with "mp_" (i.e.,
    pg.mp_active).
    '''

    ALL = '_ALL'
    DROP = '_DROP'

    def __init__(self, project, active_paths=None, stashes=None, hierarchy=None, veritesting=None,
                 veritesting_options=None, immutable=None, resilience=None, save_unconstrained=None,
                 save_unsat=None, strong_path_mapping=None):
        '''
        Initializes a new PathGroup.

        @param project: an angr.Project instance
        @param active_paths: active paths to seed the "active" stash with.
        @param stashes: a dictionary to use as the stash store
        @param hierarchy: a PathHierarchy object to use to track path reachability
        @param immutable: if True (the default), all operations will return a new
                          PathGroup. Otherwise, all operations will modify the
                          PathGroup (and return it, for consistency and chaining).
        '''
        self._project = project
        self._hierarchy = PathHierarchy(strong_path_mapping=strong_path_mapping) if hierarchy is None else hierarchy
        self._immutable = True if immutable is None else immutable
        self._veritesting = False if veritesting is None else veritesting
        self._resilience = False if resilience is None else resilience
        self._veritesting_options = { } if veritesting_options is None else veritesting_options

        # public options
        self.save_unconstrained = False if save_unconstrained is None else save_unconstrained
        self.save_unsat = False if save_unsat is None else save_unsat

        self.stashes = {
            'active': [ ] if active_paths is None else active_paths,
            'stashed': [ ],
            'pruned': [ ],
            'unsat': [ ],
            'errored': [ ],
            'deadended': [ ],
            'unconstrained': [ ]
        } if stashes is None else stashes

    @classmethod
    def call(cls, project, target, args=(), start=None, **kwargs):
        """"Calls" a function in the binary, returning a PathGroup with the call set up.

        :param project: :class:`angr.Project` instance
        :param target: address of function to call
        :param args: arguments to call the function with
        :param start: path (or paths) to start the call with
        :param **kwargs: other kwargs to pass to construct :class:`PathGroup`
        :return: a :class:`PathGroup` calling the function
        """

        fake_return_addr = project._extern_obj.get_pseudo_addr('FAKE_RETURN_ADDR')
        if not project.is_hooked(fake_return_addr):
            project.hook(fake_return_addr, CallReturn)
        cc = simuvex.DefaultCC[project.arch.name](project.arch)

        active_paths = []
        if start is None:
            active_paths.append(project.factory.path(addr=target))
        elif hasattr(start, '__iter__'):
            active_paths.extend(start)
        else:
            active_paths.append(start)

        ret_addr = claripy.BVV(fake_return_addr, project.arch.bits)

        def fix_arg(st, arg):
            if isinstance(arg, str):
                # store the string, nul-terminated, at the current heap location
                # then return a pointer to that location
                ptr = st.libc.heap_location
                st.memory.store(ptr, st.BVV(arg))
                st.memory.store(ptr + len(arg), st.BVV(0, 8))
                st.libc.heap_location += len(arg) + 1
                return ptr
            elif isinstance(arg, (tuple, list)):
                # fix the entries of the list, then store them in a
                # NULL-terminated array at the current heap location and return
                # a pointer to there
                # N.B.: uses host endianness to store entries!!! mostly useful for string arrays
                fixed_entries = [fix_arg(st, entry) for entry in arg]
                cur_ptr = start_ptr = st.libc.heap_location
                for entry in fixed_entries:
                    st.memory.store(cur_ptr, entry, endness=st.arch.memory_endness)
                    cur_ptr += entry.length

                entry_length = fixed_entries[0].length if len(fixed_entries) > 0 else st.arch.bits
                st.memory.store(cur_ptr, st.se.BVV(0, entry_length))

                st.libc.heap_location = cur_ptr + entry_length
                return start_ptr
            elif isinstance(arg, (int, long)):
                return st.se.BVV(arg, st.arch.bits)
            elif isinstance(arg, StringSpec):
                ptr = st.libc.heap_location
                arg.dump(st, ptr)
                st.libc.heap_location += len(arg)
                return ptr
            else:
                return arg

        for p in active_paths:
            p.state.ip = target
            fixed_args = [fix_arg(p.state, arg) for arg in args]
            cc.setup_callsite(p.state, ret_addr, fixed_args)

        return cls(project, active_paths=active_paths, **kwargs)

    #
    # Util functions
    #

    def copy(self):
        return PathGroup(self._project, stashes=self._copy_stashes(immutable=True), hierarchy=self._hierarchy, immutable=self._immutable)

    def _copy_stashes(self, immutable=None):
        '''
        Returns a copy of the stashes (if immutable) or the stashes themselves
        (if not immutable). Used to abstract away immutability.
        '''
        if self._immutable if immutable is None else immutable:
            return { k:list(v) for k,v in self.stashes.items() }
        else:
            return self.stashes

    def _copy_paths(self, paths):
        '''
        Returns a copy of a list of paths (if immutable) or the paths themselves
        (if not immutable). Used to abstract away immutability.
        '''
        if self._immutable:
            return [ p.copy() for p in paths ]
        else:
            return paths

    def _successor(self, new_stashes):
        '''
        Creates a new PathGroup with the provided stashes (if immutable), or sets
        the stashes (if not immutable). Used to abstract away immutability.

        @returns a PathGroup
        '''
        if '_DROP' in new_stashes:
            del new_stashes['_DROP']

        if not self._immutable:
            self.stashes = new_stashes
            return self
        else:
            return PathGroup(self._project, stashes=new_stashes, hierarchy=self._hierarchy, immutable=self._immutable)

    @staticmethod
    def _condition_to_lambda(condition, default=False):
        '''
        Translates an integer, set, or list into a lambda that checks a path address
        against the given addresses.

        @param condition: an integer, set, or list to convert to a lambda
        @param default: the default return value of the lambda (in case condition
                        is None). Default: false.
        @returns a lambda that takes a path and returns True or False
        '''
        if condition is None:
            condition = lambda p: default

        if isinstance(condition, (int, long)):
            condition = { condition }

        if isinstance(condition, (tuple, set, list)):
            addrs = set(condition)
            condition = lambda p: p.addr in addrs

        return condition

    @staticmethod
    def _filter_paths(filter_func, paths):
        '''
        Filters a sequence of paths according to a filter_func.

        @param filter_func: the filter function. Should take a path as input and
                            return a boolean
        @param paths: a sequence of paths

        @returns a tuple, with the first element the matching paths and the second
                 element the non-matching paths.
        '''
        l.debug("Filtering %d paths", len(paths))
        match = [ ]
        nomatch = [ ]

        for p in paths:
            if filter_func is None or filter_func(p):
                l.debug("... path %s matched!", p)
                match.append(p)
            else:
                l.debug("... path %s didn't match!", p)
                nomatch.append(p)

        l.debug("... returning %d matches and %d non-matches", len(match), len(nomatch))
        return match, nomatch

    def _one_step(self, stash=None, selector_func=None, successor_func=None, check_func=None, **kwargs):
        '''
        Takes a single step in a given stash.

        @param stash: the name of the stash (default: 'active').
        @param successor_func: if provided, this function is called with the path as
                               its only argument. It should return the path's
                               successors. If this is None, path.successors is used,
                               instead.
        @param selector_func: if provided, should be a lambda that takes a Path and
                              returns a boolean. If True, the path will be stepped.
                              Otherwise, it will be kept as-is.
        @param check_func: if provided, this function will be called to decide whether
                            the current path is errored or not. Path.errored will not be
                            called anymore.

        @param returns the successor PathGroup
        '''
        stash = 'active' if stash is None else stash

        new_stashes = self._copy_stashes()
        new_active = [ ]

        for a in self.stashes[stash]:
            try:
                if selector_func is not None and not selector_func(a):
                    new_active.append(a)
                    continue

                has_stashed = False # Flag that whether we have put a into a stash or not
                successors = [ ]

                veritesting_worked = False
                if self._veritesting:
                    veritesting = self._project.analyses.Veritesting(a, **self._veritesting_options)
                    if veritesting.result['result'] and veritesting.result['final_path_group']:
                        pg = veritesting.result['final_path_group']
                        pg.stash(from_stash='deviated', to_stash='active')
                        pg.stash(from_stash='successful', to_stash='active')
                        successors = pg.active
                        pg.drop(stash='active')
                        for s in pg.stashes:
                            if s not in new_stashes:
                                new_stashes[s] = []
                            new_stashes[s] += pg.stashes[s]
                        veritesting_worked = True

                if not veritesting_worked:
                    # `check_func` will not be called for Veritesting, this is
                    # intended so that we can avoid unnecessarily creating
                    # Path._run

                    if (check_func is not None and check_func(a)) or (check_func is None and a.errored):
                        # This path has error(s)!
                        if isinstance(a.error, PathUnreachableError):
                            new_stashes['pruned'].append(a)
                        else:
                            self._hierarchy.unreachable(a)
                            new_stashes['errored'].append(a)
                        has_stashed = True
                    else:
                        if successor_func is not None:
                            successors = successor_func(a)
                        else:
                            successors = a.step(**kwargs)
                            if self.save_unconstrained:
                                if 'unconstrained' not in new_stashes:
                                    new_stashes['unconstrained'] = [ ]
                                new_stashes['unconstrained'] += a.unconstrained_successors
                            if self.save_unsat:
                                if 'unsat' not in new_stashes:
                                    new_stashes['unsat'] = [ ]
                                new_stashes['unsat'] += a.unsat_successors

                self._hierarchy.add_successors(a, successors)
                if not has_stashed:
                    if len(successors) == 0:
                        new_stashes['deadended'].append(a)
                    else:
                        new_active.extend(successors)
            except (AngrError, simuvex.SimError, claripy.ClaripyError):
                if not self._resilience:
                    raise
                else:
                    l.warning("PathGroup resilience squashed an exception", exc_info=True)
                    new_stashes['errored'].append(a)

        new_stashes[stash] = new_active
        return self._successor(new_stashes)

    @staticmethod
    def _move(stashes, filter_func, src, dst):
        '''
        Moves all stashes that match the filter_func from src to dst.

        @returns a new stashes dictionary
        '''
        if dst == PathGroup.ALL:
            raise AngrPathGroupError("Can't handle '_ALL' as a target stash.")
        if src == PathGroup.DROP:
            raise AngrPathGroupError("Can't handle '_DROP' as a source stash.")

        if src == PathGroup.ALL:
            srces = [ a for a in stashes.keys() if a != dst ]
        else:
            srces = [ src ]

        matches = [ ]
        for f in srces:
            to_move, to_keep = PathGroup._filter_paths(filter_func, stashes[f])
            stashes[f] = to_keep
            matches.extend(to_move)

        if dst != PathGroup.DROP:
            if dst not in stashes:
                stashes[dst] = [ ]
            stashes[dst].extend(matches)
        return stashes

    def __repr__(self):
        s = "<PathGroup with "
        s += ', '.join(("%d %s" % (len(v),k)) for k,v in self.stashes.items() if len(v) != 0)
        s += ">"
        return s

    def __getattr__(self, k):
        if k.startswith('mp_'):
            return mulpyplexer.MP(self.stashes[k[3:]])
        else:
            return self.stashes[k]

    def __dir__(self):
        return sorted(set(self.__dict__.keys() + dir(super(PathGroup, self)) + self.stashes.keys() + [ 'mp_'+k for k in self.stashes.keys() ]))

    #
    # Interface
    #

    def apply(self, path_func=None, stash_func=None, stash=None):
        '''
        Applies a given function to a given stash.

        @param path_func: a function to apply to every path. Should take a path and
                          return a path. The returned path will take the place of the
                          old path. If the function *doesn't* return a path, the old
                          path will be used. If the function returns a list of paths,
                          they will replace the original paths.
        @param stash_func: a function to apply to the whole stash. Should take a
                           list of paths and return a list of paths. The resulting
                           list will replace the stash.

        If both path_func and stash_func are provided, path_func is applied first,
        then stash_func is applied on the results.

        @returns the resulting PathGroup
        '''
        stash = 'active' if stash is None else stash

        new_stashes = self._copy_stashes()
        new_paths = new_stashes[stash]
        if path_func is not None:
            new_new_paths = [ ]
            for p in new_paths:
                np = path_func(p)
                if isinstance(np, Path):
                    new_new_paths.append(np)
                elif isinstance(np, (list, tuple, set)):
                    new_new_paths.extend(np)
                else:
                    new_new_paths.append(p)
            new_paths = new_new_paths
        if stash_func is not None:
            new_paths = stash_func(new_paths)

        new_stashes[stash] = new_paths
        return self._successor(new_stashes)

    def split(self, stash_splitter=None, stash_ranker=None, path_ranker=None, limit=None, from_stash=None, to_stash=None):
        '''
        Split a stash of paths. The stash from_stash will be split into two
        stashes depending on the other options passed in. If to_stash is provided,
        the second stash will be written there.

        @param stash_splitter: a function that should take a list of paths and return
                               a tuple of two lists (the two resulting stashes).
        @param stash_ranker: a function that should take a list of paths and return
                             a sorted list of paths. This list will then be split
                             according to "limit".
        @param path_ranker: an alternative to stash_splitter. Paths will be sorted
                            with outputs of this function used as a key. The first
                            "limit" of them will be kept, the rest split off.
        @param limit: for use with path_ranker. The number of paths to keep. Default: 8
        @param from_stash: the stash to split (default: 'active')
        @param to_stash: the stash to write to (default: 'stashed')

        stash_splitter overrides stash_ranker, which in turn overrides path_ranker.
        If no functions are provided, the paths are simply split according to the limit.

        The sort done with path_ranker is ascending.

        @returns the resulting PathGroup
        '''

        limit = 8 if limit is None else limit
        from_stash = 'active' if from_stash is None else from_stash
        to_stash = 'stashed' if to_stash is None else to_stash

        new_stashes = self._copy_stashes()
        old_paths = new_stashes[from_stash]

        if stash_splitter is not None:
            keep, split = stash_splitter(old_paths)
        elif stash_ranker is not None:
            ranked_paths = stash_ranker(old_paths)
            keep, split = ranked_paths[:limit], ranked_paths[limit:]
        elif path_ranker is not None:
            ranked_paths = sorted(old_paths, key=path_ranker)
            keep, split = ranked_paths[:limit], ranked_paths[limit:]
        else:
            keep, split = old_paths[:limit], old_paths[limit:]

        new_stashes[from_stash] = keep
        new_stashes[to_stash] = split if to_stash in new_stashes else new_stashes[to_stash] + split
        return self._successor(new_stashes)

    def step(self, n=None, selector_func=None, step_func=None, stash=None,
             successor_func=None, until=None, check_func=None, **kwargs):
        '''
        Step a stash of paths forward.

        @param n: the number of times to step (default: 1 if "until" is not provided)
        @param selector_func: if provided, should be a lambda that takes a Path and
                              returns a boolean. If True, the path will be stepped.
                              Otherwise, it will be kept as-is.
        @param step_func: if provided, should be a lambda that takes a PathGroup and
                          returns a PathGroup. Will be called with the PathGroup at
                          at every step.
        @param stash: the name of the stash to step (default: 'active')
        @param successor_func: if provided, this function will be called with a path
                               to get its successors. Otherwise, path.successors will
                               be used.
        @param until: if provided, should be a lambda that takes a PathGroup and returns
                      True or False. Stepping will terminate when it is True.
        @param check_func: if provided, this function will be called to decide whether
                            the current path is errored or not. Path.errored will not be
                            called anymore.

        Additionally, you can pass in any of the following keyword args
        for project.factory.sim_run:

        @param jumpkind: the jumpkind of the previous exit
        @param addr an address: to execute at instead of the state's ip
        @param stmt_whitelist: a list of stmt indexes to which to confine execution
        @param last_stmt: a statement index at which to stop execution
        @param thumb: whether the block should be lifted in ARM's THUMB mode
        @param backup_state: a state to read bytes from instead of using project memory
        @param opt_level: the VEX optimization level to use
        @param insn_bytes: a string of bytes to use for the block instead of the project
        @param max_size: the maximum size of the block, in bytes
        @param num_inst: the maximum number of instructions
        @param traceflags: traceflags to be passed to VEX. Default: 0
        @returns the resulting PathGroup
        '''
        stash = 'active' if stash is None else stash
        n = n if n is not None else 1 if until is None else 100000
        pg = self

        for i in range(n):
            l.debug("Round %d: stepping %s", i, pg)

            pg = pg._one_step(stash=stash, selector_func=selector_func, successor_func=successor_func, check_func=check_func, **kwargs)
            if step_func is not None:
                pg = step_func(pg)

            if len(pg.stashes[stash]) == 0:
                l.debug("Out of paths in stash %s", stash)
                break

            if until is not None and until(pg):
                l.debug("Until function returned true")
                break

        return pg

    def prune(self, filter_func=None, from_stash=None, to_stash=None):
        '''
        Prune unsatisfiable paths from a stash.

        @param filter_func: only prune paths that match this filter
        @param from_stash: prune paths from this stash (default: 'active')
        @param to_stash: put pruned paths in this stash (default: 'pruned')

        @returns the resulting PathGroup
        '''
        to_stash = 'pruned' if to_stash is None else to_stash
        from_stash = 'active' if from_stash is None else from_stash

        to_prune, new_active = self._filter_paths(filter_func, self.stashes[from_stash])
        new_stashes = self._copy_stashes()

        for p in to_prune:
            if p.errored or not p.state.satisfiable():
                if to_stash not in new_stashes:
                    new_stashes[to_stash] = [ ]
                new_stashes[to_stash].append(p)
                self._hierarchy.unreachable(p)
            else:
                new_active.append(p)

        new_stashes[from_stash] = new_active
        return self._successor(new_stashes)

    def move(self, from_stash, to_stash, filter_func=None):
        '''
        Move paths from one stash to another.

        @param from_stash: take matching paths from this stash.
        @param to_stash: put matching paths into this stash.
        @param filter_func: stash paths that match this filter. Should be a function
                            that takes a path and returns True or False. Default: stash
                            all paths

        @returns the resulting PathGroup
        '''
        new_stashes = self._copy_stashes()
        self._move(new_stashes, filter_func, from_stash, to_stash)
        return self._successor(new_stashes)

    def stash(self, filter_func=None, from_stash=None, to_stash=None):
        '''
        Stash some paths. This is an alias for move(), with defaults for the stashes.

        @param filter_func: stash paths that match this filter. Should be a function
                            that takes a path and returns True or False. Default: stash
                            all paths
        @param from_stash: take matching paths from this stash (default: 'active')
        @param to_stash: put matching paths into this stash: (default: 'stashed')

        @returns the resulting PathGroup
        '''
        to_stash = 'stashed' if to_stash is None else to_stash
        from_stash = 'active' if from_stash is None else from_stash
        return self.move(from_stash, to_stash, filter_func=filter_func)

    def drop(self, filter_func=None, stash=None):
        '''
        Drops paths from a stash. This is an alias for move(), with defaults for the
        stashes.

        @param filter_func: drop paths that match this filter. Should be a function that
                            takes a path and returns True or False. Default:
                            drop all paths
        @param stash: drop matching paths from this stash (default: 'active')

        @returns the resulting PathGroup
        '''
        stash = 'active' if stash is None else stash
        return self.move(stash, self.DROP, filter_func=filter_func)

    def unstash(self, filter_func=None, to_stash=None, from_stash=None):
        '''
        Unstash some paths. This is an alias for move(), with defaults for the stashes.

        @param filter_func: unstash paths that match this filter. Should be a function
                            that takes a path and returns True or False. Default:
                            unstash all paths
        @param from_stash: take matching paths from this stash (default: 'stashed')
        @param to_stash: put matching paths into this stash: (default: 'active')

        @returns the resulting PathGroup
        '''
        to_stash = 'active' if to_stash is None else to_stash
        from_stash = 'stashed' if from_stash is None else from_stash
        return self.move(from_stash, to_stash, filter_func=filter_func)

    def merge(self, merge_func=None, stash=None):
        '''
        Merge the states in a given stash.

        @param stash: the stash (default: 'active')
        @param merge_func: if provided, instead of using path.merge, call this
                           function with the paths as the argument. Should return
                           the merged path.

        @returns the result PathGroup
        '''
        stash = 'active' if stash is None else stash
        to_merge = self.stashes[stash]
        not_to_merge = [ ]

        merge_groups = [ ]
        while len(to_merge) > 0:
            g, to_merge = self._filter_paths(lambda p: p.addr == to_merge[0].addr, to_merge)
            if len(g) == 1:
                not_to_merge.append(g)
            merge_groups.append(g)

        for g in merge_groups:
            try:
                m = g[0].merge(*g[1:]) if merge_func is None else merge_func(*g)
                not_to_merge.append(m)
            except simuvex.SimMergeError:
                l.warning("SimMergeError while merging %d paths", len(g), exc_info=True)
                not_to_merge.extend(g)

        new_stashes = self._copy_stashes()
        new_stashes[stash] = not_to_merge
        return self._successor(new_stashes)

    #
    # Various canned functionality
    #

    def stash_not_addr(self, addr, from_stash=None, to_stash=None):
        '''
        Stash all paths not at address addr from stash from_stash to stash to_stash.
        '''
        return self.stash(lambda p: p.addr != addr, from_stash=from_stash, to_stash=to_stash)

    def stash_addr(self, addr, from_stash=None, to_stash=None):
        '''
        Stash all paths at address addr from stash from_stash to stash to_stash.
        '''
        return self.stash(lambda p: p.addr == addr, from_stash=from_stash, to_stash=to_stash)

    def stash_addr_past(self, addr, from_stash=None, to_stash=None):
        '''
        Stash all paths containg address addr in their backtrace from stash
        from_stash to stash to_stash.
        '''
        return self.stash(lambda p: addr in p.addr_backtrace, from_stash=from_stash, to_stash=to_stash)

    def stash_not_addr_past(self, addr, from_stash=None, to_stash=None):
        '''
        Stash all paths not containg address addr in their backtrace from stash
        from_stash to stash to_stash.
        '''
        return self.stash(lambda p: addr not in p.addr_backtrace, from_stash=from_stash, to_stash=to_stash)

    def stash_all(self, from_stash=None, to_stash=None):
        '''
        Stash all paths from stash from_stash to stash to_stash.
        '''
        return self.stash(lambda p: True, from_stash=from_stash, to_stash=to_stash)

    def unstash_addr(self, addr, from_stash=None, to_stash=None):
        '''
        Untash all paths at address addr.
        '''
        return self.unstash(lambda p: p.addr == addr, from_stash=from_stash, to_stash=to_stash)

    def unstash_addr_past(self, addr, from_stash=None, to_stash=None):
        '''
        Untash all paths containing address addr in their backtrace.
        '''
        return self.unstash(lambda p: addr in p.addr_backtrace, from_stash=from_stash, to_stash=to_stash)

    def unstash_not_addr(self, addr, from_stash=None, to_stash=None):
        '''
        Untash all paths not at address addr.
        '''
        return self.unstash(lambda p: p.addr != addr, from_stash=from_stash, to_stash=to_stash)

    def unstash_not_addr_past(self, addr, from_stash=None, to_stash=None):
        '''
        Untash all paths not containing address addr in their backtrace.
        '''
        return self.unstash(lambda p: addr not in p.addr_backtrace, from_stash=from_stash, to_stash=to_stash)

    def unstash_all(self, from_stash=None, to_stash=None):
        '''
        Untash all paths.
        '''
        return self.unstash(lambda p: True, from_stash=from_stash, to_stash=to_stash)

    #
    # High-level functionality
    #

    def explore(self, stash=None, n=None, find=None, avoid=None, num_find=None, found_stash=None, avoid_stash=None):
        '''
        A replacement for the Explorer surveyor. Tick stash "stash" forward (up to n
        times or until num_find paths are found), looking for condition "find",
        avoiding condition "avoid". Stashes found paths into "found_stash' and
        avoided paths into "avoid_stash".
        '''
        find = self._condition_to_lambda(find)
        avoid = self._condition_to_lambda(avoid)
        found_stash = 'found' if found_stash is None else found_stash
        avoid_stash = 'avoid' if avoid_stash is None else avoid_stash
        num_find = 1 if num_find is None else num_find
        cur_found = len(self.stashes[found_stash]) if found_stash in self.stashes else 0

        explore_step_func = lambda pg: pg.stash(find, from_stash=stash, to_stash=found_stash) \
                                         .stash(avoid, from_stash=stash, to_stash=avoid_stash)
        until_func = lambda pg: len(pg.stashes[found_stash]) >= cur_found + num_find
        return self.step(n=n, step_func=explore_step_func, until=until_func, stash=stash)

from .path_hierarchy import PathHierarchy
from .errors import PathUnreachableError, AngrError, AngrPathGroupError
from .path import Path
from .tablespecs import StringSpec



import logging

import networkx

l = logging.getLogger("angr.pathprioritizer")

class PathPrioritizer(object):
    def __init__(self, cfg, target):
        self._cfg = cfg
        self._target = target
        self._shortest_path_length_dict = {}

        self._construct()

    def __getstate__(self):
        state = {}
        state['_shortest_path_length_dict'] = self._shortest_path_length_dict
        return state

    def _construct(self):
        g = self._cfg.graph
        bbl_dict = self._cfg.get_bbl_dict()
        assert self._target in g
        assert bbl_dict is not None

        # Reverse the bbl_dict
        bbl_key_map = {}
        for k, v in bbl_dict.items():
            bbl_key_map[v] = k

        # Reverse it
        # As SimIRSB is not copiable, we have to do it by ourselves
        reversed_graph = networkx.DiGraph()
        for a, b in g.edges():
            reversed_graph.add_edge(b, a)
        # Do a BFS from target, and save the length of shortest path to each
        # basic block
        shortest_path_length = networkx.single_source_shortest_path_length( \
                                                    reversed_graph, self._target)
        for k, v in shortest_path_length.items():
            bbl_key = bbl_key_map[k]
            self._shortest_path_length_dict[bbl_key] = v

    def get_priority(self, path):
        MAX_INT = 0xffffffff
        # Get a list of tuples
        # Each tuple looks like (a, b), where b is the function address of a
        # basic block, and a is the IRSB addr where the function is called
        l.debug("Retrieving path priority of %s...", path)
        call_stack = path.callstack
        # FIXME: For now we are only supporting level 2 context-sensitivity
        # But we shouldn't hard code this anyway
        if len(call_stack) == 0:
            tpl = (None, None, path.addr)
        else:
            tpl = call_stack[-1] + (path.addr,)
        if tpl in self._shortest_path_length_dict:
            priority = self._shortest_path_length_dict[tpl]
            l.debug("The priority is %d", priority)
            return priority
        else:
            import ipdb
            ipdb.set_trace()
            l.debug("Not in our dict")
            return MAX_INT



from ..surveyors import Slicecutor
from sleak import SleakMeta, SleakError
from angr.errors import AngrExitError
import logging

l = logging.getLogger("analysis.sleakslice")

class Sleakslice(SleakMeta):
    """
    Stack leak detection, slices through the program towards identified output
    functions.

    """

    def __init__(self, istate=None, mode=None, argc=None):
        """
        @istate: an initial state to use
        @targets: a list of target addrs to look for
        @argc: how many symbolic arguments ?
        """

        self.prepare(istate=istate, mode=mode, argc=argc)
        self.slices = []
        self.failed_targets = [] # Targets outside the CFG
        #self.found_paths = []

        #self.run()

    def run(self):

        if len(self.targets) == 0:
            raise SleakError("No targets specified")

        self.cfg = self._p.analyses.CFG(keep_input_state=True)
        self.ddg = self._p.analyses.DDG(cfg=self.cfg)
        self.cdg = self._p.analyses.CDG(cfg=self.cfg)

        for t in self.targets.values():
            l.debug("Running slice towards 0x%x" % t)
            #with self._resilience():
            try:
                r = self._run_slice(target_addr = t)
            except AngrExitError as error:  # not in the CFG
                self.failed_targets.append(t)
                l.debug(error)
                l.info("Skipping target 0x%x (%s)" % (t, self.target_name(t)))
                continue

            self.slices = self.slices + r
            self._check_found_paths()

    @property
    def terminated_paths(self):
        """
        Where did the analysis stop ?
        """
        paths=[]
        for sl in self.slices:
            paths = paths + sl.deadended + sl.cut
        return paths

    def _run_slice(self, target_addr, target_stmt = -1, begin = None):
        """
        @begin: where to start ? The default is the entry point of the program
        @target_addr: address of the destination's basic block
        @target_stmt: idx of the VEX statement in that block
        """

        target_nodes = self.cfg.get_all_nodes(target_addr)
        slices=[]

        # We create a slice per target_irsb for a given target
        for target in target_nodes:
            if target is None:
                raise AngrExitError("The CFG doesn't contain any node at "
                                    "0x%x" % target_addr)

            if begin is None:
                begin = self.ipath.addr

            bwslice = self._p.analyses.BackwardSlice(self.cfg, self.cdg, self.ddg,
                                                    target, target_stmt,
                                                    control_flow_slice=False)

            self.annocfg = bwslice.annotated_cfg(start_point=self.ipath.addr)
            slicecutor = Slicecutor(self._p, self.annocfg, start=self.ipath,
                                    targets=[target_addr])
            slicecutor.run()
            slices.append(slicecutor)
        return slices

    def _check_found_paths(self):
        """
        Iterates over all found paths to identify leaking ones
        """
        results = []
        if len(self.found_paths) > 0:
            self.reached_target = True

        # Found paths : output function reached
        for p in self.found_paths:
            r = self._check_path(p)
            if r is not None:
                results.append(r)
        self.leaks = results


